@@ -11,21 +11,18 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 #include "config.h"
 
 #include <stdio.h>
 
-#include <X11/Xlib.h>
-#include <X11/Xmd.h>
-
+#include "libs/fvwm_x11.h"
 #include "libs/fvwmlib.h"
-#include "libs/FScreen.h"
 #include "libs/Strings.h"
 #include "fvwm.h"
+#include "externs.h"
 #include "execcontext.h"
 #include "functions.h"
 #include "misc.h"
@@ -48,47 +45,48 @@ extern ewmh_atom ewmh_atom_wm_state[];
 /*
  * root
  */
-int ewmh_CurrentDesktop(EWMH_CMD_ARGS)
+int ewmh_CurrentDesktop(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
+	struct monitor	*m = monitor_get_current();
 	if (ev->xclient.data.l[0] < 0 || ev->xclient.data.l[0] > 0x7fffffff)
 	{
-		fvwm_msg(
-			WARN, "ewmh_CurrentDesktop",
-			"The application window (id %#lx)\n"
-			"  \"%s\" tried to switch to an invalid desktop (%ld)\n"
-			"  using an EWMH client message.\n"
-			"    fvwm is ignoring this request.\n",
-			fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
-			ev->xclient.data.l[0]);
+		fvwm_debug(__func__,
+			   "The application window (id %#lx)\n"
+			   "  \"%s\" tried to switch to an invalid desktop (%ld)\n"
+			   "  using an EWMH client message.\n"
+			   "    fvwm is ignoring this request.\n",
+			   fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
+			   ev->xclient.data.l[0]);
 		fvwm_msg_report_app_and_workers();
 
 		return -1;
 	}
-	goto_desk(ev->xclient.data.l[0]);
+	goto_desk(ev->xclient.data.l[0], m);
 
 	return -1;
 }
 
-int ewmh_DesktopGeometry(EWMH_CMD_ARGS)
+int ewmh_DesktopGeometry(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	char action[256];
 	long width = ev->xclient.data.l[0];
 	long height = ev->xclient.data.l[1];
 
-	width = width / Scr.MyDisplayWidth;
-	height = height / Scr.MyDisplayHeight;
+	width = width / monitor_get_all_widths();
+	height = height / monitor_get_all_heights();
 
 	if (width <= 0 || height <= 0)
 	{
-		fvwm_msg(
-			WARN, "ewmh_DesktopGeometry",
-			"The application window (id %#lx)\n"
-			"  \"%s\" tried to set an invalid desktop geometry"
-			" (%ldx%ld)\n"
-			"  using an EWMH client message.\n"
-			"    fvwm is ignoring this request.\n",
-			fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
-			ev->xclient.data.l[0], ev->xclient.data.l[1]);
+		fvwm_debug(__func__,
+			   "The application window (id %#lx)\n"
+			   "  \"%s\" tried to set an invalid desktop geometry"
+			   " (%ldx%ld)\n"
+			   "  using an EWMH client message.\n"
+			   "    fvwm is ignoring this request.\n",
+			   fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
+			   ev->xclient.data.l[0], ev->xclient.data.l[1]);
 		fvwm_msg_report_app_and_workers();
 
 		return -1;
@@ -99,53 +97,61 @@ int ewmh_DesktopGeometry(EWMH_CMD_ARGS)
 	return -1;
 }
 
-int ewmh_DesktopViewPort(EWMH_CMD_ARGS)
+int ewmh_DesktopViewPort(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
+	struct monitor	*m = (fw && fw->m) ? fw->m : monitor_get_current();
+
 	if (
 		ev->xclient.data.l[0] < 0 ||
 		ev->xclient.data.l[0] > 0x7fffffff ||
 		ev->xclient.data.l[1] < 0 ||
 		ev->xclient.data.l[1] > 0x7fffffff)
 	{
-		fvwm_msg(
-			WARN, "ewmh_DesktopViewPort",
-			"The application window (id %#lx)\n"
-			"  \"%s\" tried to switch to an invalid page"
-			" (%ldx%ld)\n"
-			"  using an EWMH client message.\n"
-			"    fvwm is ignoring this request.\n",
-			fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
-			ev->xclient.data.l[0], ev->xclient.data.l[1]);
+		fvwm_debug(__func__,
+			   "The application window (id %#lx)\n"
+			   "  \"%s\" tried to switch to an invalid page"
+			   " (%ldx%ld)\n"
+			   "  using an EWMH client message.\n"
+			   "    fvwm is ignoring this request.\n",
+			   fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
+			   ev->xclient.data.l[0], ev->xclient.data.l[1]);
 		fvwm_msg_report_app_and_workers();
 
 		return -1;
 	}
-	MoveViewport(ev->xclient.data.l[0], ev->xclient.data.l[1], 1);
+	MoveViewport(m, ev->xclient.data.l[0], ev->xclient.data.l[1], 1);
 	return -1;
 }
 
-int ewmh_NumberOfDesktops(EWMH_CMD_ARGS)
+int ewmh_NumberOfDesktops(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	int d = ev->xclient.data.l[0];
+	struct monitor	*m;
 
 	/* not a lot of sinification for fvwm */
-	if (d > 0 && (d <= ewmhc.MaxDesktops || ewmhc.MaxDesktops == 0))
-	{
-		ewmhc.NumberOfDesktops = d;
-		EWMH_SetNumberOfDesktops();
-	}
-	else
-	{
-		fvwm_msg(
-			WARN, "ewmh_NumberOfDesktops",
-			"The application window (id %#lx)\n"
-			"  \"%s\" tried to set an invalid number of desktops"
-			" (%ld)\n"
-			"  using an EWMH client message.\n"
-			"    fvwm is ignoring this request.\n",
-			fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
-			ev->xclient.data.l[0]);
-		fvwm_msg_report_app_and_workers();
+	TAILQ_FOREACH(m, &monitor_q, entry) {
+		EWMH_SetNumberOfDesktops(m);
+
+		if (d > 0 &&
+		    (d <= m->ewmhc.MaxDesktops || m->ewmhc.MaxDesktops == 0))
+		{
+			m->ewmhc.NumberOfDesktops = d;
+		}
+		else
+		{
+			fvwm_debug(__func__,
+				   "The application window (id %#lx)\n"
+				   "  \"%s\" tried to set an invalid number of desktops"
+				   " (%ld)\n"
+				   "  using an EWMH client message.\n"
+				   "    fvwm is ignoring this request.\n",
+				   fw ? FW_W(fw) : 0,
+				   fw ? fw->name.name : "(none)",
+				   ev->xclient.data.l[0]);
+			fvwm_msg_report_app_and_workers();
+		}
 	}
 
 	return -1;
@@ -155,7 +161,8 @@ int ewmh_NumberOfDesktops(EWMH_CMD_ARGS)
  * window
  */
 
-int ewmh_ActiveWindow(EWMH_CMD_ARGS)
+int ewmh_ActiveWindow(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	if (ev == NULL)
 	{
@@ -167,7 +174,8 @@ int ewmh_ActiveWindow(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_CloseWindow(EWMH_CMD_ARGS)
+int ewmh_CloseWindow(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	if (ev == NULL)
 	{
@@ -182,9 +190,9 @@ int ewmh_CloseWindow(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_MoveResizeWindow(EWMH_CMD_ARGS)
+int ewmh_MoveResizeWindow(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
-	XConfigureRequestEvent cre;
 	int do_reconfigure;
 	int win_gravity;
 	int value_mask;
@@ -221,21 +229,24 @@ int ewmh_MoveResizeWindow(EWMH_CMD_ARGS)
 	}
 	if (do_reconfigure == 1)
 	{
-		cre.value_mask = value_mask;
-		cre.x = ev->xclient.data.l[1];
-		cre.y = ev->xclient.data.l[2];
-		cre.width = ev->xclient.data.l[3];
-		cre.height = ev->xclient.data.l[4];
-		cre.window = ev->xclient.window;
-		events_handle_configure_request(cre, fw, True, win_gravity);
+		XEvent e;
+		XConfigureRequestEvent *cre = &e.xconfigurerequest;
+
+		cre->value_mask = value_mask;
+		cre->x = ev->xclient.data.l[1];
+		cre->y = ev->xclient.data.l[2];
+		cre->width = ev->xclient.data.l[3];
+		cre->height = ev->xclient.data.l[4];
+		cre->window = ev->xclient.window;
+		events_handle_configure_request(&e, fw, True, win_gravity);
 	}
 
 	return 0;
 }
 
-int ewmh_RestackWindow(EWMH_CMD_ARGS)
+int ewmh_RestackWindow(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
-	XConfigureRequestEvent cre;
 	int do_restack;
 
 	if (ev == NULL)
@@ -253,17 +264,21 @@ int ewmh_RestackWindow(EWMH_CMD_ARGS)
 	}
 	if (do_restack == 1)
 	{
-		cre.value_mask = CWSibling | CWStackMode;
-		cre.above = ev->xclient.data.l[1];
-		cre.detail = ev->xclient.data.l[2];
-		cre.window = ev->xclient.window;
-		events_handle_configure_request(cre, fw, True, ForgetGravity);
+		XEvent e;
+		XConfigureRequestEvent *cre = &e.xconfigurerequest;
+
+		cre->value_mask = CWSibling | CWStackMode;
+		cre->above = ev->xclient.data.l[1];
+		cre->detail = ev->xclient.data.l[2];
+		cre->window = ev->xclient.window;
+		events_handle_configure_request(&e, fw, True, ForgetGravity);
 	}
 
 	return 0;
 }
 
-int ewmh_WMDesktop(EWMH_CMD_ARGS)
+int ewmh_WMDesktop(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	if (ev != NULL && style == NULL)
 	{
@@ -292,16 +307,15 @@ int ewmh_WMDesktop(EWMH_CMD_ARGS)
 		}
 		else
 		{
-			fvwm_msg(
-				WARN, "ewmh_WMDesktop",
-				"The application window (id %#lx)\n"
-				"  \"%s\" tried to move to an invalid desk"
-				" (%ld)\n"
-				"  using an EWMH client message.\n"
-				"    fvwm is ignoring this request.\n",
-				fw ? FW_W(fw) : 0,
-				fw ? fw->name.name : "(none)",
-				ev->xclient.data.l[0]);
+			fvwm_debug(__func__,
+				   "The application window (id %#lx)\n"
+				   "  \"%s\" tried to move to an invalid desk"
+				   " (%ld)\n"
+				   "  using an EWMH client message.\n"
+				   "    fvwm is ignoring this request.\n",
+				   fw ? FW_W(fw) : 0,
+				   fw ? fw->name.name : "(none)",
+				   ev->xclient.data.l[0]);
 			fvwm_msg_report_app_and_workers();
 		}
 
@@ -335,11 +349,10 @@ int ewmh_WMDesktop(EWMH_CMD_ARGS)
 			return 0;
 		}
 #if DEBUG_EWMH_INIT_STATE
-		fprintf(
-			stderr, "ewmh WM_DESKTOP hint for window 0x%lx  "
-			"(%i,%lu,%u)\n", FW_W(fw),
-			HAS_EWMH_INIT_WM_DESKTOP(fw),
-			fw->ewmh_hint_desktop, val[0]);
+		fvwm_debug(__func__, "ewmh WM_DESKTOP hint for window 0x%lx  "
+			   "(%i,%lu,%u)\n", FW_W(fw),
+			   HAS_EWMH_INIT_WM_DESKTOP(fw),
+			   fw->ewmh_hint_desktop, val[0]);
 #endif
 		if (val[0] == (CARD32)-2 || val[0] == (CARD32)-1)
 		{
@@ -365,7 +378,8 @@ int ewmh_WMDesktop(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_MoveResize(EWMH_CMD_ARGS)
+int ewmh_MoveResize(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	int dir = -1;
 	int x_warp = 0;
@@ -455,7 +469,8 @@ int ewmh_MoveResize(EWMH_CMD_ARGS)
 /*
  * WM_STATE*
  */
-int ewmh_WMState(EWMH_CMD_ARGS)
+int ewmh_WMState(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	unsigned long maximize = 0;
 
@@ -498,9 +513,8 @@ int ewmh_WMState(EWMH_CMD_ARGS)
 #if DEBUG_EWMH_INIT_STATE
 		if (size != 0)
 		{
-			fprintf(
-				stderr, "Window 0x%lx has an init"
-				" _NET_WM_STATE hint\n",FW_W(fw));
+			fvwm_debug(__func__, "Window 0x%lx has an init"
+				   " _NET_WM_STATE hint\n",FW_W(fw));
 		}
 #endif
 		nitems = size / sizeof(CARD32);
@@ -549,7 +563,8 @@ int ewmh_WMState(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_WMStateFullScreen(EWMH_CMD_ARGS)
+int ewmh_WMStateFullScreen(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	if (ev == NULL && style == NULL)
 	{
@@ -564,7 +579,7 @@ int ewmh_WMStateFullScreen(EWMH_CMD_ARGS)
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(stderr,"\tFullscreen\n");
+			fvwm_debug(__func__, "\tFullscreen\n");
 		}
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
@@ -630,7 +645,8 @@ int ewmh_WMStateFullScreen(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_WMStateHidden(EWMH_CMD_ARGS)
+int ewmh_WMStateHidden(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	if (ev == NULL && style == NULL)
 	{
@@ -655,7 +671,7 @@ int ewmh_WMStateHidden(EWMH_CMD_ARGS)
 
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
-			fprintf(stderr,"\tHidden\n");
+			fvwm_debug(__func__, "\tHidden\n");
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
 		{
@@ -709,19 +725,21 @@ int ewmh_WMStateHidden(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_WMStateMaxHoriz(EWMH_CMD_ARGS)
+int ewmh_WMStateMaxHoriz(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 
 	if (ev == NULL && style == NULL)
 	{
-#if 0
-		return (IS_MAXIMIZED(fw) && !IS_EWMH_FULLSCREEN(fw));
-#else
-		/* DV: the notion of vertical/horizontal maximization does not
-		 * make any sense in fvwm, so just claim we're never maximized
+		/* If the window is maximized, but without any style, still
+		 * set the maximized status flag.
 		 */
-		return 0;
-#endif
+		bool maximized = (IS_MAXIMIZED(fw) && !IS_EWMH_FULLSCREEN(fw));
+
+		if (maximized)
+			SET_HAS_EWMH_INIT_MAXHORIZ_STATE(fw, EWMH_STATE_UNDEFINED_HINT);
+
+		return (maximized);
 	}
 
 	if (ev == NULL && style != NULL)
@@ -730,9 +748,8 @@ int ewmh_WMStateMaxHoriz(EWMH_CMD_ARGS)
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(
-				stderr, "\t Maxhoriz %i\n",
-				HAS_EWMH_INIT_MAXHORIZ_STATE(fw));
+			fvwm_debug(__func__, "\t Maxhoriz %i\n",
+				   HAS_EWMH_INIT_MAXHORIZ_STATE(fw));
 		}
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
@@ -784,19 +801,20 @@ int ewmh_WMStateMaxHoriz(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_WMStateMaxVert(EWMH_CMD_ARGS)
+int ewmh_WMStateMaxVert(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 
 	if (ev == NULL && style == NULL)
 	{
-#if 0
-		return (IS_MAXIMIZED(fw) && !IS_EWMH_FULLSCREEN(fw));
-#else
-		/* DV: the notion of vertical/horizontal maximization does not
-		 * make any sense in fvwm, so just claim we're never maximized
+		/* If the window is maximized, but without any style, still
+		 * set the maximized status flag.
 		 */
-		return 0;
-#endif
+		bool maximized = (IS_MAXIMIZED(fw) && !IS_EWMH_FULLSCREEN(fw));
+
+		if (maximized)
+			SET_HAS_EWMH_INIT_MAXVERT_STATE(fw, EWMH_STATE_UNDEFINED_HINT);
+		return (maximized);
 	}
 
 	if (ev == NULL && style != NULL)
@@ -805,9 +823,8 @@ int ewmh_WMStateMaxVert(EWMH_CMD_ARGS)
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(
-				stderr, "\t Maxvert %i\n",
-				HAS_EWMH_INIT_MAXVERT_STATE(fw));
+			fvwm_debug(__func__, "\t Maxvert %i\n",
+				   HAS_EWMH_INIT_MAXVERT_STATE(fw));
 		}
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
@@ -854,7 +871,8 @@ int ewmh_WMStateMaxVert(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_WMStateModal(EWMH_CMD_ARGS)
+int ewmh_WMStateModal(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	if (ev == NULL && style == NULL)
 	{
@@ -878,9 +896,8 @@ int ewmh_WMStateModal(EWMH_CMD_ARGS)
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(
-				stderr, "\t Modal %i\n",
-				HAS_EWMH_INIT_MODAL_STATE(fw));
+			fvwm_debug(__func__, "\t Modal %i\n",
+				   HAS_EWMH_INIT_MODAL_STATE(fw));
 		}
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
@@ -960,7 +977,8 @@ int ewmh_WMStateModal(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_WMStateShaded(EWMH_CMD_ARGS)
+int ewmh_WMStateShaded(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 
 	if (ev == NULL && style == NULL)
@@ -986,9 +1004,8 @@ int ewmh_WMStateShaded(EWMH_CMD_ARGS)
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(
-				stderr, "\t Shaded %i\n",
-				HAS_EWMH_INIT_SHADED_STATE(fw));
+			fvwm_debug(__func__, "\t Shaded %i\n",
+				   HAS_EWMH_INIT_SHADED_STATE(fw));
 		}
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
@@ -1039,7 +1056,8 @@ int ewmh_WMStateShaded(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_WMStateSkipPager(EWMH_CMD_ARGS)
+int ewmh_WMStateSkipPager(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	if (ev == NULL && style == NULL)
 	{
@@ -1062,10 +1080,9 @@ int ewmh_WMStateSkipPager(EWMH_CMD_ARGS)
 		unsigned long has_hint = any;
 #if DEBUG_EWMH_INIT_STATE
 		/*if (has_hint)*/
-		fprintf(
-			stderr, "\t Skip_Pager %lu, %i, %i\n", has_hint,
-			HAS_EWMH_INIT_SKIP_PAGER_STATE(fw),
-			DO_EWMH_IGNORE_STATE_HINTS(style));
+		fvwm_debug(__func__, "\t Skip_Pager %lu, %i, %i\n", has_hint,
+			   HAS_EWMH_INIT_SKIP_PAGER_STATE(fw),
+			   DO_EWMH_IGNORE_STATE_HINTS(style));
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
 		{
@@ -1109,7 +1126,8 @@ int ewmh_WMStateSkipPager(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_WMStateSkipTaskBar(EWMH_CMD_ARGS)
+int ewmh_WMStateSkipTaskBar(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	if (ev == NULL && style == NULL)
 	{
@@ -1132,10 +1150,10 @@ int ewmh_WMStateSkipTaskBar(EWMH_CMD_ARGS)
 		unsigned long has_hint = any;
 #if DEBUG_EWMH_INIT_STATE
 		/*if (has_hint)*/
-		fprintf(stderr,"\t Skip_Taskbar %lu, %i, %i\n",
-			has_hint,
-			HAS_EWMH_INIT_SKIP_TASKBAR_STATE(fw),
-			DO_EWMH_IGNORE_STATE_HINTS(style));
+		fvwm_debug(__func__, "\t Skip_Taskbar %lu, %i, %i\n",
+			   has_hint,
+			   HAS_EWMH_INIT_SKIP_TASKBAR_STATE(fw),
+			   DO_EWMH_IGNORE_STATE_HINTS(style));
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
 		{
@@ -1180,7 +1198,8 @@ int ewmh_WMStateSkipTaskBar(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_WMStateStaysOnTop(EWMH_CMD_ARGS)
+int ewmh_WMStateStaysOnTop(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	if (ev == NULL && style == NULL)
 	{
@@ -1207,7 +1226,7 @@ int ewmh_WMStateStaysOnTop(EWMH_CMD_ARGS)
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(stderr,"\tStaysOnTop\n");
+			fvwm_debug(__func__, "\tStaysOnTop\n");
 		}
 #endif
 		if (!DO_EWMH_USE_STACKING_HINTS(style))
@@ -1267,7 +1286,8 @@ int ewmh_WMStateStaysOnTop(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_WMStateStaysOnBottom(EWMH_CMD_ARGS)
+int ewmh_WMStateStaysOnBottom(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	if (ev == NULL && style == NULL)
 	{
@@ -1293,7 +1313,7 @@ int ewmh_WMStateStaysOnBottom(EWMH_CMD_ARGS)
 		unsigned long has_hint = any;
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
-			fprintf(stderr,"\tStaysOnBottom\n");
+			fvwm_debug(__func__, "\tStaysOnBottom\n");
 #endif
 		if (!DO_EWMH_USE_STACKING_HINTS(style))
 		{
@@ -1353,7 +1373,8 @@ int ewmh_WMStateStaysOnBottom(EWMH_CMD_ARGS)
 	return 0;
 }
 
-int ewmh_WMStateSticky(EWMH_CMD_ARGS)
+int ewmh_WMStateSticky(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 
 	if (ev == NULL && style == NULL)
@@ -1380,7 +1401,7 @@ int ewmh_WMStateSticky(EWMH_CMD_ARGS)
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(stderr,"\t Sticky\n");
+			fvwm_debug(__func__, "\t Sticky\n");
 		}
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
@@ -1422,10 +1443,13 @@ int ewmh_WMStateSticky(EWMH_CMD_ARGS)
 			execute_function_override_window(
 				NULL, NULL, "Stick on", 0, fw);
 		}
-		else
+		else if ((IS_STICKY_ACROSS_PAGES(fw) ||
+			  IS_STICKY_ACROSS_DESKS(fw)) &&
+			 (bool_arg == NET_WM_STATE_TOGGLE ||
+			  bool_arg == NET_WM_STATE_REMOVE))
 		{
 			execute_function_override_window(
-				NULL, NULL, "Stick on", 1, fw);
+				NULL, NULL, "Stick off", 1, fw);
 		}
 	}
 	return 0;
@@ -1435,7 +1459,8 @@ int ewmh_WMStateSticky(EWMH_CMD_ARGS)
  * Property Notify (_NET_WM_ICON is in ewmh_icon.c, _NET_WM_*NAME are in
  * ewmh_name)                        *
  */
-int ewmh_WMIconGeometry(EWMH_CMD_ARGS)
+int ewmh_WMIconGeometry(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	int size;
 	CARD32 *val;
@@ -1449,14 +1474,13 @@ int ewmh_WMIconGeometry(EWMH_CMD_ARGS)
 
 	if (val != NULL && size < 4 * sizeof(CARD32))
 	{
-		fvwm_msg(
-			WARN, "ewmh_WMIconGeometry",
-			"The application window (id %#lx)\n"
-			"  \"%s\" tried to set to an icon geometry via EWMH\n"
-			"  but provided only %d of the 4 values required.\n"
-			"    fvwm is ignoring this request.\n",
-			fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
-			(int)(size / sizeof(CARD32)));
+		fvwm_debug(__func__,
+			   "The application window (id %#lx)\n"
+			   "  \"%s\" tried to set to an icon geometry via EWMH\n"
+			   "  but provided only %d of the 4 values required.\n"
+			   "    fvwm is ignoring this request.\n",
+			   fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
+			   (int)(size / sizeof(CARD32)));
 		fvwm_msg_report_app_and_workers();
 		free(val);
 		val = NULL;
@@ -1498,10 +1522,14 @@ void EWMH_GetIconGeometry(FvwmWindow *fw, rectangle *icon_rect)
 	return;
 }
 
-int ewmh_WMStrut(EWMH_CMD_ARGS)
+int ewmh_WMStrut(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	int size = 0;
 	CARD32 *val;
+	struct monitor	*m;
+
+	m = (fw && fw->m) ? fw->m : monitor_get_current();
 
 	if (ev == NULL)
 	{
@@ -1529,7 +1557,7 @@ int ewmh_WMStrut(EWMH_CMD_ARGS)
 		fw->strut.right  = val[1];
 		fw->strut.top    = val[2];
 		fw->strut.bottom = val[3];
-		ewmh_ComputeAndSetWorkArea();
+		ewmh_ComputeAndSetWorkArea(m);
 	}
 	if (val[0] !=  fw->dyn_strut.left ||
 	    val[1] != fw->dyn_strut.right ||
@@ -1540,7 +1568,7 @@ int ewmh_WMStrut(EWMH_CMD_ARGS)
 		fw->dyn_strut.right  = val[1];
 		fw->dyn_strut.top    = val[2];
 		fw->dyn_strut.bottom = val[3];
-		ewmh_HandleDynamicWorkArea();
+		ewmh_HandleDynamicWorkArea(m);
 	}
 	free(val);
 
@@ -1616,7 +1644,18 @@ void EWMH_ProcessPropertyNotify(const exec_context_t *exc)
 	{
 		if (ewmh_a->action != None)
 		{
-			flush_property_notify(ewmh_a->atom, FW_W(fw));
+			flush_property_notify_stop_at_event_type(
+				ev->xproperty.atom, FW_W(fw), 0, 0);
+			if (XGetGeometry(
+				    dpy, FW_W(fw), &JunkRoot, &JunkX, &JunkY,
+				    (unsigned int*)&JunkWidth,
+				    (unsigned int*)&JunkHeight,
+				    (unsigned int*)&JunkBW,
+				    (unsigned int*)&JunkDepth) == 0)
+			{
+				/* Window does not exist anymore. */
+				return;
+			}
 			ewmh_a->action(fw, ev, NULL, 0);
 		}
 	}
