@@ -10,8 +10,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /* ---------------------------- included header files ---------------------- */
@@ -19,13 +18,14 @@
 #include "config.h"
 
 #include <stdio.h>
-#include <X11/keysym.h>
 
+#include "libs/fvwm_x11.h"
 #include "libs/Parse.h"
 #include "libs/Bindings.h"
 #include "libs/charmap.h"
 #include "libs/Flocale.h"
 #include "libs/wcontext.h"
+#include "libs/FEvent.h"
 #include "fvwm.h"
 #include "fvwm.h"
 #include "externs.h"
@@ -242,9 +242,9 @@ static void parse_menu_action(
 		{
 			if (suffix[0] != 0 || count[0] != 0)
 			{
-				fvwm_msg(ERR, "parse_menu_action",
-					 "invalid MenuMoveCursor arguments "
-					 "'%s'", options);
+				fvwm_debug(__func__,
+					   "invalid MenuMoveCursor arguments "
+					   "'%s'", options);
 				*saction = SA_NONE;
 				break;
 			}
@@ -275,9 +275,9 @@ static void parse_menu_action(
 		}
 		else
 		{
-			fvwm_msg(ERR, "parse_menu_action",
-				 "invalid MenuMoveCursor arguments '%s'",
-				 options);
+			fvwm_debug(__func__,
+				   "invalid MenuMoveCursor arguments '%s'",
+				   options);
 			*saction = SA_NONE;
 			break;
 		}
@@ -311,9 +311,9 @@ static void parse_menu_action(
 			}
 			else
 			{
-				fvwm_msg(ERR, "parse_menu_action",
-					 "invalid MenuScroll arguments '%s'",
-					 options);
+				fvwm_debug(__func__,
+					   "invalid MenuScroll arguments '%s'",
+					   options);
 				*saction = SA_NONE;
 				break;
 			}
@@ -327,9 +327,8 @@ static void parse_menu_action(
 		*ret_cmd = options;
 		break;
 	default:
-		fvwm_msg(
-			ERR, "parse_menu_action", "unknown action '%s'",
-			action);
+		fvwm_debug(__func__, "unknown action '%s'",
+			   action);
 		*saction = SA_NONE;
 	}
 
@@ -402,6 +401,7 @@ int menu_binding(
 {
 	Binding *rmlist;
 	int rc;
+	Bool dummy;
 
 	if (menu_bindings == NULL)
 	{
@@ -410,18 +410,16 @@ int menu_binding(
 	rmlist = NULL;
 	if (~(~context | C_MENU | C_TITLE | C_MENU_ITEM | C_SIDEBAR) != 0)
 	{
-		fvwm_msg(
-			ERR, "menu_binding",
-			"invalid context in combination with menu context.");
+		fvwm_debug(__func__,
+			   "invalid context in combination with menu context.");
 
 		return 1;
 	}
 	if (menu_style != NULL)
 	{
 		/*!!! fixme - make either match a menu style or a menu name */
-		fvwm_msg(
-			ERR, "menu_binding", "a window name may not be"
-			" specified without a menu context.");
+		fvwm_debug(__func__, "a window name may not be"
+			   " specified without a menu context.");
 
 		return 1;
 	}
@@ -430,7 +428,7 @@ int menu_binding(
 	 */
 	/* BEGIN remove */
 	CollectBindingList(
-		dpy, menu_bindings, &rmlist, type, STROKE_ARG(NULL)
+		dpy, menu_bindings, &rmlist, &dummy, type,
 		button, keysym, modifier, context, menu_style);
 	if (rmlist != NULL)
 	{
@@ -442,10 +440,9 @@ int menu_binding(
 	{
 		/* Warn if Mouse n M N - occurs without removing any binding.
 		 The user most likely want Mouse n MT A - instead. */
-		fvwm_msg(
-			WARN, "menu_binding",
-			"The syntax for disabling the tear off button has "
-			"changed.");
+		fvwm_debug(__func__,
+			   "The syntax for disabling the tear off button has "
+			   "changed.");
 	}
 	if (strcmp(action,"-") == 0)
 	{
@@ -454,10 +451,9 @@ int menu_binding(
 	/* END remove */
 	if ((modifier & AnyModifier) && (modifier & (~AnyModifier)))
 	{
-		fvwm_msg(
-			WARN, "menu_binding", "Binding specified AnyModifier"
-			" and other modifers too. Excess modifiers are"
-			" ignored.");
+		fvwm_debug(__func__, "Binding specified AnyModifier"
+			   " and other modifiers too. Excess modifiers are"
+			   " ignored.");
 		modifier = AnyModifier;
 	}
 	/* Warn about Mouse n M N TearOff. */
@@ -465,13 +461,13 @@ int menu_binding(
 		keysym == 0 && button != 0 && modifier == 0 &&
 		strcasecmp(action,"tearoff") == 0 && context == C_MENU)
 	{
-		fvwm_msg(OLD, "menu_binding",
-			 "The syntax for disabling the tear off button has "
-			 "changed. The TearOff action is no longer possible "
-			 "in menu bindings.");
+		fvwm_debug(__func__,
+			   "The syntax for disabling the tear off button has "
+			   "changed. The TearOff action is no longer possible "
+			   "in menu bindings.");
 	}
 	rc = AddBinding(
-		dpy, menu_bindings, type, STROKE_ARG(NULL) button, keysym,
+		dpy, menu_bindings, type, button, keysym,
 		NULL, modifier, context, (void *)action, NULL, menu_style);
 
 	return rc;
@@ -590,9 +586,8 @@ void menu_shortcuts(
 		}
 		else
 		{
-			fvwm_msg(
-				ERR, "menu_shortcuts", "can't get geometry of"
-				" menu %s", MR_NAME(mr));
+			fvwm_debug(__func__, "can't get geometry of"
+				   " menu %s", MR_NAME(mr));
 		}
 	}
 
@@ -978,9 +973,9 @@ void menu_shortcuts(
 				&JunkWidth, &menu_height,
 				&JunkBW, &JunkDepth))
 		{
-			fvwm_msg(
-				ERR, "menu_shortcuts",
-				"can't get geometry of menu %s", MR_NAME(mr));
+			fvwm_debug(__func__,
+				   "can't get geometry of menu %s",
+				   MR_NAME(mr));
 			return;
 		}
 		if (do_skip_section)
@@ -1074,15 +1069,15 @@ void menu_shortcuts(
 
 					if (
 						menu_y + menu_height >
-						MR_SCREEN_HEIGHT(mr))
+						MR_SCREEN_Y(mr) + MR_SCREEN_HEIGHT(mr))
 					{
 						FWarpPointer(
 							dpy, 0, 0, 0, 0, 0, 0,
 							0,
 							MR_SCREEN_HEIGHT(mr) -
 							menu_y - menu_height);
-						menu_y = MR_SCREEN_HEIGHT(mr) -
-							menu_height;
+						menu_y = MR_SCREEN_Y(mr) + MR_SCREEN_HEIGHT(mr) -
+							 menu_height;
 					}
 				}
 				if (old_y != menu_y)
@@ -1118,20 +1113,19 @@ void menu_shortcuts(
 			    &JunkWidth, &menu_height,
 			    &JunkBW, &JunkDepth))
 		{
-			fvwm_msg(
-				ERR, "menu_shortcuts",
-				"can't get geometry of menu %s", MR_NAME(mr));
+			fvwm_debug(__func__,
+				   "can't get geometry of menu %s",
+				   MR_NAME(mr));
 			return;
 		}
-		if (menu_y < 0 || menu_y + menu_height > MR_SCREEN_HEIGHT(mr))
+		if (menu_y < 0 || menu_y + menu_height > MR_SCREEN_Y(mr) + MR_SCREEN_HEIGHT(mr))
 		{
 			menu_y = (menu_y < 0) ?
-				0 : MR_SCREEN_HEIGHT(mr) - menu_height;
+				 0 : MR_SCREEN_Y(mr) + MR_SCREEN_HEIGHT(mr) - menu_height;
 			pmret->rc = MENU_NEWITEM_MOVEMENU;
 			*ret_menu_x = menu_x;
 			*ret_menu_y = menu_y;
 		}
 	}
-
 	return;
 }
