@@ -10,16 +10,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
-
-/*
- * This module is all original code
- * by Rob Nation
- * Copyright 1993, Robert Nation
- *     You may use this code for any purpose, as long as the original
- *     copyright remains in the source code and all documentation
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /*
@@ -35,11 +26,11 @@
 
 #include "libs/ftime.h"
 #include "libs/fvwmlib.h"
-#include "libs/FScreen.h"
 #include "libs/ColorUtils.h"
 #include "libs/Parse.h"
 #include "libs/Strings.h"
 #include "libs/wild.h"
+#include "libs/FEvent.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "functions.h"
@@ -130,10 +121,9 @@ make_new_vpacket(unsigned char *body, unsigned long event_type,
 		bodylen += addlen;
 		if (bodylen >= FvwmPacketMaxSize_byte)
 		{
-			fvwm_msg(
-				ERR, "make_new_vpacket",
-				"packet too long %ld %ld", (long)bodylen,
-				(long)FvwmPacketMaxSize_byte);
+			fvwm_debug(__func__,
+				   "packet too long %ld %ld", (long)bodylen,
+				   (long)FvwmPacketMaxSize_byte);
 			break;
 		}
 		if (arglen > 0)
@@ -169,10 +159,10 @@ make_new_vpacket(unsigned char *body, unsigned long event_type,
 		}
 		else
 		{
-			fvwm_msg(
-				ERR, "make_new_vpacket",
-				"can not handle arglen %ld, please contact"
-				" fvwm-workers@fvwm.org. aborting...", arglen);
+			fvwm_debug(__func__,
+				   "can not handle arglen %ld, please contact"
+				   " fvwm-workers@fvwm.org. aborting...",
+				   arglen);
 			abort();
 		}
 	}
@@ -300,7 +290,7 @@ action_flags *__get_allowed_actions(const FvwmWindow *fw)
   as a dummy to preserve alignment of the other fields in the
   old packet: we should drop this before the next release.
 */
-#define CONFIGARGS(_fw) 33,				\
+#define CONFIGARGS(_fw) 35,				\
 		(unsigned long)(-sizeof(Window)),	\
 		&FW_W(*(_fw)),				\
 		(unsigned long)(-sizeof(Window)),	\
@@ -318,6 +308,10 @@ action_flags *__get_allowed_actions(const FvwmWindow *fw)
 		(unsigned long)(0),			\
 		&(*(_fw))->Desk,			\
 		(unsigned long)(0),			\
+		&(*(_fw))->m->si->rr_output,		\
+		(unsigned long)(sizeof(unsigned long)),\
+		(*(_fw))->m->si->name,			\
+		(unsigned long)(0),			\
 		&(*(_fw))->layer,			\
 		(unsigned long)(0),			\
 		&(*(_fw))->hints.base_width,		\
@@ -404,7 +398,7 @@ static unsigned long *make_named_packet(
 		*len = FvwmPacketMaxSize;
 	}
 
-	body = (unsigned long *)safemalloc(*len * sizeof(unsigned long));
+	body = fxmalloc(*len * sizeof(unsigned long));
 	/* Zero out end of memory to avoid uninit memory access. */
 	body[*len-1] = 0;
 
@@ -463,6 +457,41 @@ void BroadcastName(
 	return;
 }
 
+void BroadcastMonitorList(fmodule *this)
+{
+	char		*name;
+	struct monitor	*m;
+	fmodule_list_itr moditr;
+	fmodule *module;
+
+	module_list_itr_init(&moditr);
+
+	while ((module = module_list_itr_next(&moditr)) != NULL) {
+		TAILQ_FOREACH(m, &monitor_q, entry) {
+			if (m->flags & MONITOR_DISABLED)
+				continue;
+			xasprintf(&name, "Monitor %s %d %d %d %d %d %d %d %d %d %d %d %d",
+				m->si->name,
+				(int)m->si->rr_output,
+				m == monitor_get_current(),
+				monitor_get_all_widths(),
+				monitor_get_all_heights(),
+				m->virtual_scr.Vx,
+				m->virtual_scr.Vy,
+				m->virtual_scr.VxMax,
+				m->virtual_scr.VyMax,
+				m->si->x,
+				m->si->y,
+				m->si->w,
+				m->si->h
+			);
+
+			SendName(module, M_CONFIG_INFO, 0, 0, 0, name);
+			free(name);
+		}
+	}
+}
+
 void BroadcastWindowIconNames(FvwmWindow *fw, Bool window, Bool icon)
 {
 	if (window)
@@ -626,18 +655,6 @@ void BroadcastConfigInfoString(char *string)
 	return;
 }
 
-
-/*
- * Broadcasts the state of Xinerama support to all modules as M_CONFIG_INFO.
- */
-void broadcast_xinerama_state(void)
-{
-	BroadcastConfigInfoString((char *)FScreenGetConfiguration());
-
-	return;
-}
-
-
 /*
  * Broadcasts the ignored modifiers to all modules as M_CONFIG_INFO.
  */
@@ -657,6 +674,7 @@ void module_input_execute(struct fmodule_input *input)
 	XEvent e;
 	const exec_context_t *exc;
 	exec_context_changes_t ecc;
+	int flags;
 
 	memset(&e, 0, sizeof(e));
 	if (XFindContext(dpy, input->window, FvwmContext,
@@ -702,13 +720,14 @@ void module_input_execute(struct fmodule_input *input)
 	fev_fake_event(&e);
 	ecc.type = EXCT_MODULE;
 	ecc.w.w = input->window;
+	flags = (input->window == None) ? 0 : FUNC_DONT_DEFER;
 	ecc.w.wcontext = GetContext(NULL, ecc.w.fw, &e, &(input->window));
 	ecc.x.etrigger = &e;
 	ecc.m.module = input->module;
 	exc = exc_create_context(
 		&ecc, ECC_TYPE | ECC_ETRIGGER | ECC_FW | ECC_W | ECC_WCONTEXT |
 		ECC_MODULE);
-	execute_function(NULL, exc, input->command, 0);
+	execute_function(NULL, exc, input->command, flags);
 	exc_destroy_context(exc);
 	module_input_discard(input);
 
@@ -723,7 +742,6 @@ void module_input_enqueue(struct fmodule_input *input)
 		return;
 	}
 
-	DBUG("module_input_enqueue", input->command);
 	fqueue_add_at_end(&cqueue, (void*)input);
 }
 
@@ -748,7 +766,6 @@ void ExecuteCommandQueue(void)
 		/* execute and destroy */
 		if (input->command)
 		{
-			DBUG("ExecuteCommandQueue", input->command);
 			module_input_execute(input);
 		}
 		else
@@ -859,118 +876,137 @@ void CMD_Send_WindowList(F_CMD_ARGS)
 {
 	FvwmWindow *t;
 	fmodule *mod = exc->m.module;
+	struct monitor	*m;
 
 	if (mod == NULL)
 	{
 		return;
 	}
-	SendPacket(mod, M_NEW_DESK, 1, (long)Scr.CurrentDesk);
-	SendPacket(
-		mod, M_NEW_PAGE, 7, (long)Scr.Vx, (long)Scr.Vy,
-		(long)Scr.CurrentDesk, (long)Scr.MyDisplayWidth,
-		(long)Scr.MyDisplayHeight,
-		(long)((Scr.VxMax / Scr.MyDisplayWidth) + 1),
-		(long)((Scr.VyMax / Scr.MyDisplayHeight) + 1));
 
-	if (Scr.Hilite != NULL)
-	{
+	/* TA: 2020-01-09:  We send this for *ALL* configured monitors.
+	 *
+	 * Modules can filter out those monitor packets they're not interested
+	 * in.
+	 */
+	TAILQ_FOREACH(m, &monitor_q, entry) {
+		SendPacket(mod, M_NEW_DESK, 2, (long)m->virtual_scr.CurrentDesk,
+			(long)m->si->rr_output);
 		SendPacket(
-			mod, M_FOCUS_CHANGE, 5, (long)FW_W(Scr.Hilite),
-			(long)FW_W_FRAME(Scr.Hilite), (unsigned long)True,
-			(long)Scr.Hilite->hicolors.fore,
-			(long)Scr.Hilite->hicolors.back);
-	}
-	else
-	{
-		SendPacket(
-			mod, M_FOCUS_CHANGE, 5, 0, 0, (unsigned long)True,
-			(long)GetColor(DEFAULT_FORE_COLOR),
-			(long)GetColor(DEFAULT_BACK_COLOR));
-	}
-	if (Scr.DefaultIcon != NULL)
-	{
-		SendName(mod, M_DEFAULTICON, 0, 0, 0, Scr.DefaultIcon);
-	}
+			mod, M_NEW_PAGE, 8, (long)m->virtual_scr.Vx, (long)m->virtual_scr.Vy,
+			(long)m->virtual_scr.CurrentDesk,
+			(long) monitor_get_all_widths(),
+			(long) monitor_get_all_heights(),
+			(long)((m->virtual_scr.VxMax / monitor_get_all_widths()) + 1),
+			(long)((m->virtual_scr.VyMax / monitor_get_all_heights()) + 1),
+			(long)m->si->rr_output);
 
-	for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
-	{
-		SendConfig(mod,M_CONFIGURE_WINDOW,t);
-		SendName(
-			mod, M_WINDOW_NAME, FW_W(t), FW_W_FRAME(t),
-			(unsigned long)t, t->name.name);
-		SendName(
-			mod, M_ICON_NAME, FW_W(t), FW_W_FRAME(t),
-			(unsigned long)t, t->icon_name.name);
-		SendName(
-			mod, M_VISIBLE_NAME, FW_W(t), FW_W_FRAME(t),
-			(unsigned long)t, t->visible_name);
-		SendName(
-			mod, MX_VISIBLE_ICON_NAME, FW_W(t), FW_W_FRAME(t),
-			(unsigned long)t,t->visible_icon_name);
-		if (t->icon_bitmap_file != NULL
-		    && t->icon_bitmap_file != Scr.DefaultIcon)
+		if (Scr.Hilite != NULL)
 		{
-			SendName(
-				mod, M_ICON_FILE, FW_W(t), FW_W_FRAME(t),
-				(unsigned long)t, t->icon_bitmap_file);
+			SendPacket(
+				mod, M_FOCUS_CHANGE, 5, (long)FW_W(Scr.Hilite),
+				(long)FW_W_FRAME(Scr.Hilite), (unsigned long)True,
+				(long)Scr.Hilite->hicolors.fore,
+				(long)Scr.Hilite->hicolors.back);
+		}
+		else
+		{
+			SendPacket(
+				mod, M_FOCUS_CHANGE, 5, 0, 0, (unsigned long)True,
+				(long)GetColor(DEFAULT_FORE_COLOR),
+				(long)GetColor(DEFAULT_BACK_COLOR));
+		}
+		if (Scr.DefaultIcon != NULL)
+		{
+			SendName(mod, M_DEFAULTICON, 0, 0, 0, Scr.DefaultIcon);
 		}
 
-		SendName(
-			mod, M_RES_CLASS, FW_W(t), FW_W_FRAME(t),
-			(unsigned long)t, t->class.res_class);
-		SendName(
-			mod, M_RES_NAME, FW_W(t), FW_W_FRAME(t),
-			(unsigned long)t, t->class.res_name);
-
-		if (IS_ICONIFIED(t) && !IS_ICON_UNMAPPED(t))
+		for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
 		{
-			rectangle r;
-			Bool rc;
+			if ((monitor_mode == MONITOR_TRACKING_M) && t->m != m)
+				continue;
 
-			rc = get_visible_icon_geometry(t, &r);
-			if (rc == True)
+			SendConfig(mod,M_CONFIGURE_WINDOW,t);
+			SendName(
+				mod, M_WINDOW_NAME, FW_W(t), FW_W_FRAME(t),
+				(unsigned long)t, t->name.name);
+			SendName(
+				mod, M_ICON_NAME, FW_W(t), FW_W_FRAME(t),
+				(unsigned long)t, t->icon_name.name);
+			SendName(
+				mod, M_VISIBLE_NAME, FW_W(t), FW_W_FRAME(t),
+				(unsigned long)t, t->visible_name);
+			SendName(
+				mod, MX_VISIBLE_ICON_NAME, FW_W(t), FW_W_FRAME(t),
+				(unsigned long)t,t->visible_icon_name);
+			if (t->icon_bitmap_file != NULL
+			    && t->icon_bitmap_file != Scr.DefaultIcon)
+			{
+				SendName(
+					mod, M_ICON_FILE, FW_W(t), FW_W_FRAME(t),
+					(unsigned long)t, t->icon_bitmap_file);
+			}
+
+			SendName(
+				mod, M_RES_CLASS, FW_W(t), FW_W_FRAME(t),
+				(unsigned long)t, t->class.res_class);
+			SendName(
+				mod, M_RES_NAME, FW_W(t), FW_W_FRAME(t),
+				(unsigned long)t, t->class.res_name);
+
+			if (IS_ICONIFIED(t) && !IS_ICON_UNMAPPED(t))
+			{
+				rectangle r;
+				Bool rc;
+
+				rc = get_visible_icon_geometry(t, &r);
+				if (rc == True)
+				{
+					SendPacket(
+						mod, M_ICONIFY, 7, (long)FW_W(t),
+						(long)FW_W_FRAME(t), (unsigned long)t,
+						(long)r.x, (long)r.y,
+						(long)r.width, (long)r.height);
+				}
+			}
+			if ((IS_ICONIFIED(t))&&(IS_ICON_UNMAPPED(t)))
 			{
 				SendPacket(
 					mod, M_ICONIFY, 7, (long)FW_W(t),
 					(long)FW_W_FRAME(t), (unsigned long)t,
-					(long)r.x, (long)r.y,
-					(long)r.width, (long)r.height);
+					(long)0, (long)0, (long)0, (long)0);
+			}
+			if (FMiniIconsSupported && t->mini_icon != NULL)
+			{
+				SendFvwmPicture(
+					mod, M_MINI_ICON, FW_W(t), FW_W_FRAME(t),
+					(unsigned long)t, t->mini_icon,
+					t->mini_pixmap_file);
 			}
 		}
-		if ((IS_ICONIFIED(t))&&(IS_ICON_UNMAPPED(t)))
-		{
-			SendPacket(
-				mod, M_ICONIFY, 7, (long)FW_W(t),
-				(long)FW_W_FRAME(t), (unsigned long)t,
-				(long)0, (long)0, (long)0, (long)0);
-		}
-		if (FMiniIconsSupported && t->mini_icon != NULL)
-		{
-			SendFvwmPicture(
-				mod, M_MINI_ICON, FW_W(t), FW_W_FRAME(t),
-				(unsigned long)t, t->mini_icon,
-				t->mini_pixmap_file);
-		}
-	}
 
-	if (Scr.Hilite == NULL)
-	{
-		BroadcastPacket(
-			M_FOCUS_CHANGE, 5, (long)0, (long)0,
-			(unsigned long)True,
-			(long)GetColor(DEFAULT_FORE_COLOR),
-			(long)GetColor(DEFAULT_BACK_COLOR));
-	}
-	else
-	{
-		BroadcastPacket(
-			M_FOCUS_CHANGE, 5, (long)FW_W(Scr.Hilite),
-			(long)FW_W(Scr.Hilite), (unsigned long)True,
-			(long)Scr.Hilite->hicolors.fore,
-			(long)Scr.Hilite->hicolors.back);
+		if (Scr.Hilite == NULL)
+		{
+			BroadcastPacket(
+				M_FOCUS_CHANGE, 5, (long)0, (long)0,
+				(unsigned long)True,
+				(long)GetColor(DEFAULT_FORE_COLOR),
+				(long)GetColor(DEFAULT_BACK_COLOR));
+		}
+		else
+		{
+			BroadcastPacket(
+				M_FOCUS_CHANGE, 5, (long)FW_W(Scr.Hilite),
+				(long)FW_W(Scr.Hilite), (unsigned long)True,
+				(long)Scr.Hilite->hicolors.fore,
+				(long)Scr.Hilite->hicolors.back);
+		}
+
+		/* If we're tracking just the global monitor, then stop here
+		 * -- essentialy, we only need to send this informatio once.
+		 */
+		if (monitor_mode == MONITOR_TRACKING_G)
+			break;
 	}
 
 	SendPacket(mod, M_END_WINDOWLIST, 0);
-
-	return;
 }
