@@ -10,8 +10,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 /*
   This file is strongly based on the corresponding files from
@@ -28,8 +27,8 @@
 #endif
 #include <signal.h>
 #include <fcntl.h>
-#include <X11/Xatom.h>
 
+#include "libs/fvwm_x11.h"
 #include "libs/fvwmlib.h"
 #include "libs/FSMlib.h"
 #include "libs/Strings.h"
@@ -48,12 +47,12 @@
 #include "virtual.h"
 #include "geometry.h"
 #include "move_resize.h"
+#include "infostore.h"
 
 /* ---------------------------- local definitions -------------------------- */
 
 /*#define FVWM_SM_DEBUG_PROTO*/
 /*#define FVWM_SM_DEBUG_WINMATCH*/
-/*#define FVWM_SM_DEBUG_FILES*/
 
 /* ---------------------------- local macros ------------------------------- */
 
@@ -84,6 +83,7 @@ typedef struct _match
 	int x_max, y_max, w_max, h_max;
 	int width_defect_max, height_defect_max;
 	int max_x_offset, max_y_offset;
+	struct monitor *m;
 	int desktop;
 	int layer;
 	int default_layer;
@@ -101,15 +101,15 @@ typedef struct _match
 static char *previous_sm_client_id = NULL;
 static char *sm_client_id = NULL;
 static Bool sent_save_done = 0;
-static char *realStateFilename = NULL;
-static Bool goingToRestart = False;
+static char *real_state_filename = NULL;
+static Bool going_to_restart = False;
 static FIceIOErrorHandler prev_handler;
 static FSmcConn sm_conn = NULL;
 
 static int num_match = 0;
 static Match *matches = NULL;
 static Bool does_file_version_match = False;
-static Bool doPreserveState = True;
+static Bool do_preserve_state = True;
 
 /* ---------------------------- exported variables (globals) --------------- */
 
@@ -123,9 +123,11 @@ char *duplicate(const char *s)
 	int l;
 	char *r;
 
+	/* TA:  FIXME!  Use xasprintf() */
+
 	if (!s) return NULL;
 	l = strlen(s);
-	r = (char *) safemalloc (sizeof(char)*(l+1));
+	r = fxmalloc (sizeof(char)*(l+1));
 	strncpy(r, s, l+1);
 
 	return r;
@@ -133,16 +135,13 @@ char *duplicate(const char *s)
 
 static char *get_version_string(void)
 {
-	/* migo (14-Mar-2001): it is better to manually update a version string
-	 * in the stable branch, otherwise saving sessions becomes useless */
-	return CatString3(VERSION, ", ", __DATE__);
-	/* return "2.6-0"; */
+	return (VERSION);
 }
 
 static char *unspace_string(const char *str)
 {
 	static const char *spaces = " \t\n";
-	char *tr_str = CatString2(str, NULL);
+	char *tr_str = fxstrdup(str);
 	int i;
 
 	if (!tr_str)
@@ -172,38 +171,57 @@ static char *unspace_string(const char *str)
 static int
 SaveGlobalState(FILE *f)
 {
+	struct monitor	*m;
+
 	fprintf(f, "[GLOBAL]\n");
-	fprintf(f, "  [DESKTOP] %i\n", Scr.CurrentDesk);
-	fprintf(f, "  [VIEWPORT] %i %i %i %i\n",
-		Scr.Vx, Scr.Vy, Scr.VxMax, Scr.VyMax);
-	fprintf(f, "  [SCROLL] %i %i %i %i %i\n",
-		Scr.EdgeScrollX, Scr.EdgeScrollY, Scr.ScrollDelay,
-		!!(Scr.flags.do_edge_wrap_x), !!(Scr.flags.do_edge_wrap_y));
+	TAILQ_FOREACH(m, &monitor_q, entry) {
+		fprintf(f, "  [MONITOR] %i\n", (int)m->si->rr_output);
+		fprintf(f, "      [DESKTOP] %i\n", m->virtual_scr.CurrentDesk);
+		fprintf(f, "      [VIEWPORT] %i %i %i %i\n",
+			m->virtual_scr.Vx, m->virtual_scr.Vy,
+			m->virtual_scr.VxMax, m->virtual_scr.VyMax);
+		fprintf(f, "      [SCROLL] %i %i %i %i %i\n",
+			m->virtual_scr.EdgeScrollX, m->virtual_scr.EdgeScrollY,
+			Scr.ScrollDelay,
+			!!(Scr.flags.do_edge_wrap_x),
+			!!(Scr.flags.do_edge_wrap_y));
+	}
 	fprintf(f, "  [MISC] %i %i %i\n",
 		Scr.ClickTime, Scr.ColormapFocus, Scr.ColorLimit);
 	fprintf(
 		f, "  [STYLE] %i %i\n", Scr.gs.do_emulate_mwm,
 		Scr.gs.do_emulate_win);
 
+	if (get_metainfo_length() > 0) {
+		MetaInfo *mi = get_metainfo(), *mi_i;
+
+		fprintf(f, "  [INFOSTORE]\n");
+		for (mi_i = mi; mi_i; mi_i = mi_i->next) {
+			fprintf(f, "    [KEY] %s\n", mi_i->key);
+			fprintf(f, "    [VALUE] %s\n", mi_i->value);
+		}
+	}
+
+
 	return 1;
 }
 
-static void setRealStateFilename(char *filename)
+static void set_real_state_filename(char *filename)
 {
 	if (!SessionSupport)
 	{
 		return;
 	}
-	if (realStateFilename)
+	if (real_state_filename)
 	{
-		free(realStateFilename);
+		free(real_state_filename);
 	}
-	realStateFilename = safestrdup(filename);
+	real_state_filename = fxstrdup(filename);
 
 	return;
 }
 
-static char *getUniqueStateFilename(void)
+static char *get_unique_state_filename(void)
 {
 	const char *path = getenv("SM_SAVE_DIR");
 	char *filename;
@@ -235,7 +253,7 @@ static char *getUniqueStateFilename(void)
 	{
 		return NULL;
 	}
-	filename = safestrdup(CatString2(path, "/.fs-XXXXXX"));
+	xasprintf(&filename, "%s%s", path, "/.fs-XXXXXX");
 	fd = fvwm_mkstemp(filename);
 	if (fd == -1)
 	{
@@ -361,12 +379,11 @@ static Bool VerifyVersionInfo(char *filename)
 			}
 			else
 			{
-				fvwm_msg(
-					ERR, "VerifyVersionInfo",
-					"State file version (%s) does not"
-					" match the current version (%s), "
-					"state file is ignored.", s1,
-					current_v);
+				fvwm_debug(__func__,
+					   "State file version (%s) does not"
+					   " match the current version (%s), "
+					   "state file is ignored.", s1,
+					   current_v);
 				break;
 			}
 		}
@@ -394,6 +411,7 @@ SaveWindowStates(FILE *f)
 	FvwmWindow *ewin;
 	rectangle save_g;
 	rectangle ig;
+	struct monitor	*m = monitor_get_current();
 	int i;
 	int layer;
 
@@ -437,6 +455,14 @@ SaveWindowStates(FILE *f)
 				/* No client id and no WM_COMMAND, the client
 				 * cannot be managed by the sessiom manager
 				 * skip it! */
+				/* TA:  20110611 - But actually, this breaks
+				 * those applications which don't set the
+				 * WM_COMMAND XAtom anymore.  The ICCCM
+				 * deprecated this at version 2.0 -- and its
+				 * lack of existence here shouldn't be a
+				 * problem.  Let newer session managers handle
+				 * the error if it even matters.
+				 */
 				if (!Restarting)
 				{
 					if (wm_command)
@@ -444,7 +470,11 @@ SaveWindowStates(FILE *f)
 						XFreeStringList(wm_command);
 						wm_command = NULL;
 					}
-					continue;
+					/* TA: 20110611 - But see above.  We
+					 * no longer skip clients who don't
+					 * set this legacy field.
+					 */
+					/* continue; */
 				}
 			}
 		}
@@ -490,8 +520,11 @@ SaveWindowStates(FILE *f)
 					wm_command_count);
 				for (i = 0; i < wm_command_count; i++)
 				{
-					fprintf(f, " %s",
-						unspace_string(wm_command[i]));
+					char *us;
+
+					us = unspace_string(wm_command[i]);
+					fprintf(f, " %s", us);
+					free(us);
 				}
 				fprintf(f, "\n");
 			}
@@ -508,8 +541,8 @@ SaveWindowStates(FILE *f)
 			&ewin->g.normal);
 		if (IS_STICKY_ACROSS_PAGES(ewin))
 		{
-			save_g.x -= Scr.Vx;
-			save_g.y -= Scr.Vy;
+			save_g.x -= m->virtual_scr.Vx;
+			save_g.y -= m->virtual_scr.Vy;
 		}
 		get_visible_icon_geometry(ewin, &ig);
 		fprintf(
@@ -519,10 +552,11 @@ SaveWindowStates(FILE *f)
 			ewin->g.max.x, ewin->g.max.y, ewin->g.max.width,
 			ewin->g.max.height, ewin->g.max_defect.width,
 			ewin->g.max_defect.height,
-			ig.x + ((!is_icon_sticky_across_pages) ? Scr.Vx : 0),
-			ig.y + ((!is_icon_sticky_across_pages) ? Scr.Vy : 0),
+			ig.x + ((!is_icon_sticky_across_pages) ? m->virtual_scr.Vx : 0),
+			ig.y + ((!is_icon_sticky_across_pages) ? m->virtual_scr.Vy : 0),
 			ewin->hints.win_gravity,
 			ewin->g.max_offset.x, ewin->g.max_offset.y);
+		fprintf(f, "  [MONITOR] %i\n", (int)ewin->m->si->rr_output);
 		fprintf(f, "  [DESK] %i\n", ewin->Desk);
 		/* set the layer to the default layer if the layer has been
 		 * set by an ewmh hint */
@@ -603,12 +637,15 @@ static Bool matchWin(FvwmWindow *w, Match *m)
 				{
 					for (i = 0; i < wm_command_count; i++)
 					{
-						if (strcmp(unspace_string(
-								   wm_command[i]),
-							   m->wm_command[i])!=0)
+						char *us;
+
+						us = unspace_string(wm_command[i]);
+						if (strcmp(us, m->wm_command[i])!=0)
 						{
+							free(us);
 							break;
 						}
+						free(us);
 					}
 
 					if (i == wm_command_count)
@@ -627,43 +664,43 @@ static Bool matchWin(FvwmWindow *w, Match *m)
 	} /* if client_id's agree */
 
 #ifdef FVWM_SM_DEBUG_WINMATCH
-	fprintf(stderr,
-		"\twin(%s, %s, %s, %s, %s,",
-		w->class.res_name, w->class.res_class, w->name.name,
-		(client_id)? client_id:"(null)",
-		(window_role)? window_role:"(null)");
+	fvwm_debug(__func__,
+		   "\twin(%s, %s, %s, %s, %s,",
+		   w->class.res_name, w->class.res_class, w->name.name,
+		   (client_id)? client_id:"(null)",
+		   (window_role)? window_role:"(null)");
 	if (wm_command)
 	{
 		for (i = 0; i < wm_command_count; i++)
 		{
-			fprintf(stderr," %s", wm_command[i]);
+			fvwm_debug(__func__, " %s", wm_command[i]);
 		}
-		fprintf(stderr,",");
+		fvwm_debug(__func__, ",");
 	}
 	else
 	{
-		fprintf(stderr," no_wmc,");
+		fvwm_debug(__func__, " no_wmc,");
 	}
-	fprintf(stderr," %d)", IS_NAME_CHANGED(w));
-	fprintf(stderr,"\n[%d]", found);
-	fprintf(stderr,
-		"\tmat(%s, %s, %s, %s, %s,",
-		m->res_name, m->res_class, m->wm_name,
-		(m->client_id)?m->client_id:"(null)",
-		(m->window_role)?m->window_role:"(null)");
+	fvwm_debug(__func__, " %d)", IS_NAME_CHANGED(w));
+	fvwm_debug(__func__, "\n[%d]", found);
+	fvwm_debug(__func__,
+		   "\tmat(%s, %s, %s, %s, %s,",
+		   m->res_name, m->res_class, m->wm_name,
+		   (m->client_id)?m->client_id:"(null)",
+		   (m->window_role)?m->window_role:"(null)");
 	if (m->wm_command)
 	{
 		for (i = 0; i < m->wm_command_count; i++)
 		{
-			fprintf(stderr," %s", m->wm_command[i]);
+			fvwm_debug(__func__, " %s", m->wm_command[i]);
 		}
-		fprintf(stderr,",");
+		fvwm_debug(__func__, ",");
 	}
 	else
 	{
-		fprintf(stderr," no_wmc,");
+		fvwm_debug(__func__, " no_wmc,");
 	}
-	fprintf(stderr," %d)\n\n", IS_NAME_CHANGED(m));
+	fvwm_debug(__func__, " %d)\n\n", IS_NAME_CHANGED(m));
 #endif
 
 	if (client_id)
@@ -682,7 +719,7 @@ static Bool matchWin(FvwmWindow *w, Match *m)
 	return found;
 }
 
-static int saveStateFile(char *filename)
+static int save_state_file(char *filename)
 {
 	FILE *f;
 	int success;
@@ -701,33 +738,24 @@ static int saveStateFile(char *filename)
 	fprintf(f, "# Normally, you must never delete this file,"
 		" it will be auto-deleted.\n\n");
 
-	if (SessionSupport && goingToRestart)
+	if (SessionSupport && going_to_restart)
 	{
-		fprintf(f, "[REAL_STATE_FILENAME] %s\n", realStateFilename);
-		goingToRestart = False;  /* not needed */
+		fprintf(f, "[REAL_STATE_FILENAME] %s\n", real_state_filename);
+		going_to_restart = False;  /* not needed */
 	}
 
-	success = doPreserveState?
-		SaveVersionInfo(f) && SaveWindowStates(f) && SaveGlobalState(f):
-		1;
-	doPreserveState = True;
+	success = do_preserve_state
+		? SaveVersionInfo(f) && SaveWindowStates(f) && SaveGlobalState(f)
+		: 1;
+	do_preserve_state = True;
 	if (fclose(f) != 0)
 		return 0;
 
-#ifdef FVWM_SM_DEBUG_FILES
-	system(CatString3(
-		       "mkdir -p /tmp/fs-save; cp ", filename,
-		       " /tmp/fs-save"));
-#endif
-#if defined(FVWM_SM_DEBUG_PROTO) || defined(FVWM_SM_DEBUG_FILES)
-	fprintf(stderr, "[FVWM_SMDEBUG] Saving %s\n", filename);
-#endif
-
 	return success;
 }
 
 static void
-setSmProperties(FSmcConn sm_conn, char *filename, char hint)
+set_sm_properties(FSmcConn sm_conn, char *filename, char hint)
 {
 	FSmProp prop1, prop2, prop3, prop4, prop5, prop6, prop7, *props[7];
 	FSmPropValue prop1val, prop2val, prop3val, prop4val, prop7val;
@@ -735,7 +763,7 @@ setSmProperties(FSmcConn sm_conn, char *filename, char hint)
 	char *user_id;
 	char screen_num[32];
 	int numVals, i, priority = 30;
-	Bool xsmDetected = False;
+	Bool is_xsm_detected = False;
 
 	if (!SessionSupport)
 	{
@@ -743,8 +771,10 @@ setSmProperties(FSmcConn sm_conn, char *filename, char hint)
 	}
 
 #ifdef FVWM_SM_DEBUG_PROTO
-	fprintf(stderr, "[FVWM_SMDEBUG][setSmProperties] state filename: %s%s\n",
-		(filename)? filename:"(null)", sm_conn? "": " - not connected");
+	fvwm_debug(__func__,
+		   "[FVWM_SMDEBUG][set_sm_properties] state filename: %s%s\n",
+		   filename ? filename : "(null)",
+		   sm_conn ? "" : " - not connected");
 #endif
 
 	if (!sm_conn)
@@ -866,9 +896,9 @@ setSmProperties(FSmcConn sm_conn, char *filename, char hint)
 
 		prop7.name = FSmDiscardCommand;
 
-		xsmDetected = StrEquals(getenv("SESSION_MANAGER_NAME"), "xsm");
+		is_xsm_detected = StrEquals(getenv("SESSION_MANAGER_NAME"), "xsm");
 
-		if (xsmDetected)
+		if (is_xsm_detected)
 		{
 			/* the protocol spec says that the discard command
 			   should be LISTofARRAY8 on posix systems, but xsm
@@ -904,7 +934,7 @@ setSmProperties(FSmcConn sm_conn, char *filename, char hint)
 	props[2] = &prop3;
 	props[3] = &prop4;
 	props[4] = &prop5;
-
+	SUPPRESS_UNUSED_VAR_WARNING(props);
 	if (filename)
 	{
 		props[5] = &prop6;
@@ -912,7 +942,7 @@ setSmProperties(FSmcConn sm_conn, char *filename, char hint)
 		FSmcSetProperties (sm_conn, 7, props);
 
 		free ((char *) prop6.vals);
-		if (!xsmDetected)
+		if (!is_xsm_detected)
 		{
 			free ((char *) prop7.vals);
 		}
@@ -936,16 +966,16 @@ callback_save_yourself2(FSmcConn sm_conn, FSmPointer client_data)
 		return;
 	}
 
-	filename = getUniqueStateFilename();
+	filename = get_unique_state_filename();
 #ifdef FVWM_SM_DEBUG_PROTO
-	fprintf(stderr, "[FVWM_SMDEBUG][callback_save_yourself2]\n");
+	fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_save_yourself2]\n");
 #endif
 
-	success = saveStateFile(filename);
+	success = save_state_file(filename);
 	if (success)
 	{
-		setSmProperties(sm_conn, filename, FSmRestartIfRunning);
-		setRealStateFilename(filename);
+		set_sm_properties(sm_conn, filename, FSmRestartIfRunning);
+		set_real_state_filename(filename);
 	}
 	free(filename);
 
@@ -965,17 +995,17 @@ callback_save_yourself(FSmcConn sm_conn, FSmPointer client_data,
 	}
 
 #ifdef FVWM_SM_DEBUG_PROTO
-	fprintf(stderr, "[FVWM_SMDEBUG][callback_save_yourself] "
-		"(save=%d, shut=%d, intr=%d, fast=%d)\n",
-		save_style, shutdown, interact_style, fast);
+	fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_save_yourself] "
+		   "(save=%d, shut=%d, intr=%d, fast=%d)\n",
+		   save_style, shutdown, interact_style, fast);
 #endif
 
 	if (save_style == FSmSaveGlobal)
 	{
 		/* nothing to do */
 #ifdef FVWM_SM_DEBUG_PROTO
-		fprintf(stderr, "[FVWM_SMDEBUG][callback_save_yourself] "
-		"Global Save type ... do nothing\n");
+		fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_save_yourself] "
+			   "Global Save type ... do nothing\n");
 #endif
 		FSmcSaveYourselfDone (sm_conn, True);
 		sent_save_done = 1;
@@ -983,8 +1013,8 @@ callback_save_yourself(FSmcConn sm_conn, FSmPointer client_data,
 
 	}
 #ifdef FVWM_SM_DEBUG_PROTO
-	fprintf(stderr, "[FVWM_SMDEBUG][callback_save_yourself] "
-		"Both or Local save type, going to phase 2 ...");
+	fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_save_yourself] "
+		   "Both or Local save type, going to phase 2 ...");
 #endif
 	if (!FSmcRequestSaveYourselfPhase2(
 		    sm_conn, callback_save_yourself2, NULL))
@@ -992,13 +1022,13 @@ callback_save_yourself(FSmcConn sm_conn, FSmPointer client_data,
 		FSmcSaveYourselfDone (sm_conn, False);
 		sent_save_done = 1;
 #ifdef FVWM_SM_DEBUG_PROTO
-		fprintf(stderr, " failed!\n");
+		fvwm_debug(__func__, " failed!\n");
 #endif
 	}
 	else
 	{
 #ifdef FVWM_SM_DEBUG_PROTO
-		fprintf(stderr, " OK\n");
+		fvwm_debug(__func__, " OK\n");
 #endif
 		sent_save_done = 0;
 	}
@@ -1016,7 +1046,7 @@ callback_die(FSmcConn sm_conn, FSmPointer client_data)
 	}
 
 #ifdef FVWM_SM_DEBUG_PROTO
-	fprintf(stderr, "[FVWM_SMDEBUG][callback_die]\n");
+	fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_die]\n");
 #endif
 
 	if (FSmcCloseConnection(sm_conn, 0, NULL) != FSmcClosedNow)
@@ -1040,7 +1070,7 @@ callback_save_complete(FSmcConn sm_conn, FSmPointer client_data)
 		return;
 	}
 #ifdef FVWM_SM_DEBUG_PROTO
-	fprintf(stderr, "[FVWM_SMDEBUG][callback_save_complete]\n");
+	fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_save_complete]\n");
 #endif
 
 	return;
@@ -1055,7 +1085,7 @@ callback_shutdown_cancelled(FSmcConn sm_conn, FSmPointer client_data)
 	}
 
 #ifdef FVWM_SM_DEBUG_PROTO
-	fprintf(stderr, "[FVWM_SMDEBUG][callback_shutdown_cancelled]\n");
+	fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_shutdown_cancelled]\n");
 #endif
 
 	if (!sent_save_done)
@@ -1112,7 +1142,9 @@ LoadGlobalState(char *filename)
 	FILE *f;
 	char s[4096], s1[4096];
 	/* char s2[256]; */
-	int i1, i2, i3, i4, i5;
+	char *is_key = NULL, *is_value = NULL;
+	int n, i1, i2, i3, i4;
+	struct monitor	*mon = NULL;
 
 	if (!does_file_version_match)
 	{
@@ -1129,12 +1161,14 @@ LoadGlobalState(char *filename)
 
 	while (fgets(s, sizeof(s), f))
 	{
+		n = 0;
+
 		i1 = 0;
 		i2 = 0;
 		i3 = 0;
 		i4 = 0;
-		i5 = 0;
-		sscanf(s, "%4000s", s1);
+
+		sscanf(s, "%4000s%n", s1, &n);
 		/* If we are restarting, [REAL_STATE_FILENAME] points
 		 * to the file containing the true session state. */
 		if (SessionSupport && !strcmp(s1, "[REAL_STATE_FILENAME]"))
@@ -1142,14 +1176,19 @@ LoadGlobalState(char *filename)
 			/* migo: temporarily (?) moved to
 			   LoadWindowStates (trick for gnome-session)
 			   sscanf(s, "%*s %s", s2);
-			   setFSmProperties (sm_conn, s2, FSmRestartIfRunning);
-			   setRealStateFilename(s2);
+			   set_sm_properties(sm_conn, s2, FSmRestartIfRunning);
+			   set_real_state_filename(s2);
 			*/
 		}
+		else if (!strcmp(s1, "[MONITOR]"))
+		{
+			sscanf(s, "%*s %i", &i2);
+			mon = monitor_by_output(i2);
+		}
 		else if (!strcmp(s1, "[DESKTOP]"))
 		{
 			sscanf(s, "%*s %i", &i1);
-			goto_desk(i1);
+			goto_desk(i1, mon);
 		}
 		else if (!strcmp(s1, "[VIEWPORT]"))
 		{
@@ -1161,7 +1200,36 @@ LoadGlobalState(char *filename)
 			 Scr.VxMax = i3;
 			 Scr.VyMax = i4;
 			*/
-			MoveViewport(i1, i2, True);
+			MoveViewport(mon, i1, i2, True);
+		}
+		else if (!strcmp(s1, "[KEY]"))
+		{
+			char *s2;
+			s2 = s + n;
+			if (*s2 != 0)
+			{
+				s2++;
+			}
+			sscanf(s2, "%[^\n]", s1);
+			is_key = fxstrdup(s1);
+		}
+		else if (!strcmp(s1, "[VALUE]"))
+		{
+			char *s2;
+			s2 = s + n;
+			if (*s2 != 0)
+			{
+				s2++;
+			}
+			sscanf(s2, "%[^\n]", s1);
+			is_value = fxstrdup(s1);
+
+			if (is_key != NULL && is_value != NULL)
+				insert_metainfo(is_key, is_value);
+
+			free(is_key);
+			free(is_value);
+			is_key = is_value = NULL;
 		}
 #if 0
 		/* migo (08-Dec-1999): we don't want to eliminate config yet */
@@ -1173,8 +1241,8 @@ LoadGlobalState(char *filename)
 			 * changed rc files. */
 			if (!strcmp(s1, "[SCROLL]"))
 			{
-				sscanf(s, "%*s %i %i %i %i %i", &i1,
-				       &i2, &i3, &i4, &i5);
+				sscanf(s, "%*s %i %i %i %i ", &i1,
+				       &i2, &i3, &i4);
 				Scr.EdgeScrollX = i1;
 				Scr.EdgeScrollY = i2;
 				Scr.ScrollDelay = i3;
@@ -1243,21 +1311,12 @@ LoadWindowStates(char *filename)
 	{
 		return;
 	}
-	setRealStateFilename(filename);
+	set_real_state_filename(filename);
 	if ((f = fopen(filename, "r")) == NULL)
 	{
 		return;
 	}
 
-#if defined(FVWM_SM_DEBUG_PROTO) ||  defined(FVWM_SM_DEBUG_FILES)
-	fprintf(stderr, "[FVWM_SMDEBUG] Loading %s\n", filename);
-#endif
-#ifdef FVWM_SM_DEBUG_FILES
-	system(CatString3(
-		       "mkdir -p /tmp/fs-load; cp ", filename,
-		       " /tmp/fs-load"));
-#endif
-
 	while (fgets(s, sizeof(s), f))
 	{
 		n = 0;
@@ -1266,15 +1325,15 @@ LoadWindowStates(char *filename)
 		    !strcmp(s1, "[REAL_STATE_FILENAME]"))
 		{
 			sscanf(s, "%*s %s", s1);
-			setSmProperties (sm_conn, s1, FSmRestartIfRunning);
-			setRealStateFilename(s1);
+			set_sm_properties(sm_conn, s1, FSmRestartIfRunning);
+			set_real_state_filename(s1);
 		}
 		else if (!strcmp(s1, "[CLIENT]"))
 		{
 			sscanf(s, "%*s %lx", &w);
 			num_match++;
-			matches = (Match *)saferealloc(
-				(void *)matches, sizeof(Match) * num_match);
+			matches = fxrealloc(
+				(void *)matches, sizeof(Match), num_match);
 			matches[num_match - 1].win = w;
 			matches[num_match - 1].client_id = NULL;
 			matches[num_match - 1].res_name = NULL;
@@ -1289,12 +1348,13 @@ LoadWindowStates(char *filename)
 			matches[num_match - 1].h = 100;
 			matches[num_match - 1].x_max = 0;
 			matches[num_match - 1].y_max = 0;
-			matches[num_match - 1].w_max = Scr.MyDisplayWidth;
-			matches[num_match - 1].h_max = Scr.MyDisplayHeight;
+			matches[num_match - 1].w_max = monitor_get_all_widths();
+			matches[num_match - 1].h_max = monitor_get_all_heights();
 			matches[num_match - 1].width_defect_max = 0;
 			matches[num_match - 1].height_defect_max = 0;
 			matches[num_match - 1].icon_x = 0;
 			matches[num_match - 1].icon_y = 0;
+			matches[num_match - 1].m = NULL;
 			matches[num_match - 1].desktop = 0;
 			matches[num_match - 1].layer = 0;
 			matches[num_match - 1].default_layer = 0;
@@ -1302,6 +1362,19 @@ LoadWindowStates(char *filename)
 			       sizeof(window_flags));
 			matches[num_match - 1].used = 0;
 		}
+		else if (!strcmp(s1, "[MONITOR]"))
+		{
+			struct monitor *m;
+
+			if (num_match == 0) {
+				num_match++;
+				matches = fxrealloc(
+					(void *)matches, sizeof(Match), num_match);
+			}
+			sscanf(s, "%*s %i", &pos);
+			m = monitor_by_output(pos);
+			matches[num_match - 1].m = m;
+		}
 		else if (!strcmp(s1, "[GEOMETRY]"))
 		{
 			sscanf(s, "%*s %i %i %i %i %i %i %i %i %i %i %i %i"
@@ -1418,7 +1491,7 @@ LoadWindowStates(char *filename)
 			sscanf(s, "%*s %i%n",
 			       &matches[num_match - 1].wm_command_count, &pos);
 			matches[num_match - 1].wm_command = (char **)
-				safemalloc(
+				fxmalloc(
 					matches[num_match - 1].
 					wm_command_count * sizeof (char *));
 			for (i = 0;
@@ -1450,6 +1523,7 @@ MatchWinToSM(
 	initial_window_options_t *win_opts)
 {
 	int i;
+	struct monitor *m = monitor_get_current();
 
 	if (!does_file_version_match)
 	{
@@ -1501,7 +1575,7 @@ MatchWinToSM(
 				{
 					free_window_names(ewin, True, False);
 					ewin->name.name = matches[i].wm_name;
-					setup_visible_name(ewin, False);
+					setup_visible_names(ewin, 1);
 				}
 			}
 			SET_NAME_CHANGED(ewin,IS_NAME_CHANGED(&(matches[i])));
@@ -1529,10 +1603,11 @@ MatchWinToSM(
 				      IS_ICON_STICKY_ACROSS_PAGES(
 					      &(matches[i]))))
 				{
-					win_opts->initial_icon_x -= Scr.Vx;
-					win_opts->initial_icon_y -= Scr.Vy;
+					win_opts->initial_icon_x -= m->virtual_scr.Vx;
+					win_opts->initial_icon_y -= m->virtual_scr.Vy;
 				}
 			}
+			ewin->m = matches[i].m;
 			ewin->g.normal.x = matches[i].x;
 			ewin->g.normal.y = matches[i].y;
 			ewin->g.normal.width = matches[i].w;
@@ -1551,7 +1626,7 @@ MatchWinToSM(
 			SET_STICKY_ACROSS_DESKS(
 				ewin, IS_STICKY_ACROSS_DESKS(&(matches[i])));
 			ewin->Desk = (IS_STICKY_ACROSS_DESKS(ewin)) ?
-				Scr.CurrentDesk : matches[i].desktop;
+				m->virtual_scr.CurrentDesk : matches[i].desktop;
 			set_layer(ewin, matches[i].layer);
 			set_default_layer(ewin, matches[i].default_layer);
 			ewin->placed_by_button = matches[i].placed_by_button;
@@ -1584,31 +1659,31 @@ MatchWinToSM(
 }
 
 void
-RestartInSession (char *filename, Bool isNative, Bool _doPreserveState)
+RestartInSession (char *filename, Bool is_native, Bool _do_preserve_state)
 {
-	doPreserveState = _doPreserveState;
+	do_preserve_state = _do_preserve_state;
 
-	if (SessionSupport && sm_conn && isNative)
+	if (SessionSupport && sm_conn && is_native)
 	{
-		goingToRestart = True;
+		going_to_restart = True;
 
-		saveStateFile(filename);
-		setSmProperties(sm_conn, filename, FSmRestartImmediately);
+		save_state_file(filename);
+		set_sm_properties(sm_conn, filename, FSmRestartImmediately);
 
-		MoveViewport(0, 0, False);
+		MoveViewport(monitor_get_current(), 0, 0, False);
 		Reborder();
 
 		CloseICCCM2();
 		XCloseDisplay(dpy);
 
-		if (!FSmcCloseConnection(sm_conn, 0, NULL) != FSmcClosedNow)
+		if ((!FSmcCloseConnection(sm_conn, 0, NULL)) != FSmcClosedNow)
 		{
 			/* go a head any way ? */
 		}
 
 #ifdef  FVWM_SM_DEBUG_PROTO
-		fprintf(stderr, "[FVWM_SMDEBUG]: Exiting, now SM must "
-			"restart us.\n");
+		fvwm_debug(__func__, "[FVWM_SMDEBUG]: Exiting, now SM must "
+			   "restart us.\n");
 #endif
 		/* Close all my pipes */
 		module_kill_all();
@@ -1616,7 +1691,7 @@ RestartInSession (char *filename, Bool isNative, Bool _doPreserveState)
 		exit(0); /* let the SM restart us */
 	}
 
-	saveStateFile(filename);
+	save_state_file(filename);
 	/* return and let Done restart us */
 
 	return;
@@ -1648,7 +1723,6 @@ SessionInit(void)
 		return;
 	}
 
-	context = NULL;
 	InstallIOErrorHandler();
 
 	callbacks.save_yourself.callback = callback_save_yourself;
@@ -1659,6 +1733,7 @@ SessionInit(void)
 		callbacks.die.client_data =
 		callbacks.save_complete.client_data =
 		callbacks.shutdown_cancelled.client_data = (FSmPointer) NULL;
+	SUPPRESS_UNUSED_VAR_WARNING(context);
 	sm_conn = FSmcOpenConnection(
 		NULL, &context, FSmProtoMajor, FSmProtoMinor,
 		FSmcSaveYourselfProcMask | FSmcDieProcMask |
@@ -1672,28 +1747,27 @@ SessionInit(void)
 		*/
 		if (previous_sm_client_id)
 		{
-			fvwm_msg(
-				ERR, "SessionInit",
-				"While connecting to session manager:\n%s.",
-				error_string_ret);
+			fvwm_debug(__func__,
+				   "While connecting to session manager:\n%s.",
+				   error_string_ret);
 		}
 		sm_fd = -1;
 #ifdef FVWM_SM_DEBUG_PROTO
-		fprintf(stderr,"[FVWM_SMDEBUG] No SM connection\n");
+		fvwm_debug(__func__, "[FVWM_SMDEBUG] No SM connection\n");
 #endif
 	}
 	else
 	{
 		sm_fd = FIceConnectionNumber(FSmcGetIceConnection(sm_conn));
 #ifdef FVWM_SM_DEBUG_PROTO
-		fprintf(stderr,"[FVWM_SMDEBUG] Connectecd to a SM\n");
+		fvwm_debug(__func__, "[FVWM_SMDEBUG] Connectecd to a SM\n");
 #endif
-		setInitFunctionName(0, "SessionInitFunction");
-		setInitFunctionName(1, "SessionRestartFunction");
-		setInitFunctionName(2, "SessionExitFunction");
+		set_init_function_name(0, "SessionInitFunction");
+		set_init_function_name(1, "SessionRestartFunction");
+		set_init_function_name(2, "SessionExitFunction");
 		/* basically to restet our restart style hint after a
 		 * restart */
-		setSmProperties(sm_conn, NULL, FSmRestartIfRunning);
+		set_sm_properties(sm_conn, NULL, FSmRestartIfRunning);
 	}
 
 	return;
@@ -1710,7 +1784,8 @@ ProcessICEMsgs(void)
 	}
 
 #ifdef FVWM_SM_DEBUG_PROTO
-	fprintf(stderr,"[FVWM_SMDEBUG][ProcessICEMsgs] %i\n", (int)sm_fd);
+	fvwm_debug(__func__, "[FVWM_SMDEBUG][ProcessICEMsgs] %i\n",
+		   (int)sm_fd);
 #endif
 	if (sm_fd < 0)
 	{
@@ -1719,8 +1794,8 @@ ProcessICEMsgs(void)
 	status = FIceProcessMessages(FSmcGetIceConnection(sm_conn), NULL, NULL);
 	if (status == FIceProcessMessagesIOError)
 	{
-		fvwm_msg(ERR, "ProcessICEMSGS",
-			 "Connection to session manager lost\n");
+		fvwm_debug(__func__,
+			   "Connection to session manager lost\n");
 		sm_conn = NULL;
 		sm_fd = -1;
 	}
@@ -1735,42 +1810,42 @@ ProcessICEMsgs(void)
  * Alternative implementation may use unix signals, but this does not work
  * with all session managers (also must suppose that SM runs locally).
  */
-int getSmPid(void)
+int get_sm_pid(void)
 {
-	const char *sessionManagerVar = getenv("SESSION_MANAGER");
-	const char *smPidStrPtr;
-	int smPid = 0;
+	const char *session_manager_var = getenv("SESSION_MANAGER");
+	const char *sm_pid_str_ptr;
+	int sm_pid = 0;
 
 	if (!SessionSupport)
 	{
 		return 0;
 	}
 
-	if (!sessionManagerVar)
+	if (!session_manager_var)
 	{
 		return 0;
 	}
-	smPidStrPtr = strchr(sessionManagerVar, ',');
-	if (!smPidStrPtr)
+	sm_pid_str_ptr = strchr(session_manager_var, ',');
+	if (!sm_pid_str_ptr)
 	{
 		return 0;
 	}
-	while (smPidStrPtr > sessionManagerVar && isdigit(*(--smPidStrPtr)))
+	while (sm_pid_str_ptr > session_manager_var && isdigit(*(--sm_pid_str_ptr)))
 	{
 		/* nothing */
 	}
-	while (isdigit(*(++smPidStrPtr)))
+	while (isdigit(*(++sm_pid_str_ptr)))
 	{
-		smPid = smPid * 10 + *smPidStrPtr - '0';
+		sm_pid = sm_pid * 10 + *sm_pid_str_ptr - '0';
 	}
 
-	return smPid;
+	return sm_pid;
 }
 
 /*
- * quitSession - hopefully shutdowns the session
+ * quit_session - hopefully shutdowns the session
  */
-Bool quitSession(void)
+Bool quit_session(void)
 {
 	if (!SessionSupport)
 	{
@@ -1789,16 +1864,16 @@ Bool quitSession(void)
 	/* migo: xsm does not support RequestSaveYourself, but supports
 	 * signals: */
 	/*
-	  int smPid = getSmPid();
-	  if (!smPid) return False;
-	  return kill(smPid, SIGTERM) == 0? True: False;
+	  int sm_pid = get_sm_pid();
+	  if (!sm_pid) return False;
+	  return kill(sm_pid, SIGTERM) == 0 ? True : False;
 	*/
 }
 
 /*
- * saveSession - hopefully saves the session
+ * save_session - hopefully saves the session
  */
-Bool saveSession(void)
+Bool save_session(void)
 {
 	if (!SessionSupport)
 	{
@@ -1817,16 +1892,16 @@ Bool saveSession(void)
 	/* migo: xsm does not support RequestSaveYourself, but supports
 	 * signals: */
 	/*
-	  int smPid = getSmPid();
-	  if (!smPid) return False;
-	  return kill(smPid, SIGUSR1) == 0? True: False;
+	  int sm_pid = get_sm_pid();
+	  if (!sm_pid) return False;
+	  return kill(sm_pid, SIGUSR1) == 0 ? True : False;
 	*/
 }
 
 /*
- * saveQuitSession - hopefully saves and shutdowns the session
+ * save_quit_session - hopefully saves and shutdowns the session
  */
-Bool saveQuitSession(void)
+Bool save_quit_session(void)
 {
 	if (!SessionSupport)
 	{
@@ -1846,9 +1921,9 @@ Bool saveQuitSession(void)
 	/* migo: xsm does not support RequestSaveYourself, but supports
 	 * signals: */
 	/*
-	  if (saveSession() == False) return False;
+	  if (save_session() == False) return False;
 	  sleep(3);  / * doesn't work anyway * /
-	  if (quitSession() == False) return False;
+	  if (quit_session() == False) return False;
 	  return True;
 	*/
 }
@@ -1857,21 +1932,21 @@ Bool saveQuitSession(void)
 
 void CMD_QuitSession(F_CMD_ARGS)
 {
-	quitSession();
+	quit_session();
 
 	return;
 }
 
 void CMD_SaveSession(F_CMD_ARGS)
 {
-	saveSession();
+	save_session();
 
 	return;
 }
 
 void CMD_SaveQuitSession(F_CMD_ARGS)
 {
-	saveQuitSession();
+	save_quit_session();
 
 	return;
 }
