@@ -1,10 +1,10 @@
 /* -*-c-*- */
 
-#ifndef _STYLE_
-#define _STYLE_
+#ifndef FVWM_STYLE_H
+#define FVWM_STYLE_H
 
-/* The default title in case the user doesn't supply one. */
-#define DEFAULT_TITLE_FORMAT "%n"
+#include "fvwm.h"
+#include "update.h"
 
 /* access to the special flags of a style */
 /* call these with a pointer to a style_flags struct */
@@ -168,10 +168,6 @@
 	((c).s.do_iconify_window_groups)
 #define S_SET_DO_ICONIFY_WINDOW_GROUPS(c,x) \
 	((c).s.do_iconify_window_groups = !!(x))
-#define S_DO_IGNORE_GNOME_HINTS(c) \
-	((c).s.do_ignore_gnome_hints)
-#define S_SET_DO_IGNORE_GNOME_HINTS(c,x) \
-	((c).s.do_ignore_gnome_hints = !!(x))
 #define S_DO_IGNORE_ICON_BOXES(c) \
 	((c).s.do_ignore_icon_boxes)
 #define S_SET_DO_IGNORE_ICON_BOXES(c,x) \
@@ -426,12 +422,10 @@
 	((s).mini_icon_name)
 #define SSET_MINI_ICON_NAME(s,x) \
 	((s).mini_icon_name = (x))
-#ifdef USEDECOR
 #define SGET_DECOR_NAME(s) \
 	((s).decor_name)
 #define SSET_DECOR_NAME(s,x) \
 	((s).decor_name = (x))
-#endif
 #define SGET_FORE_COLOR_NAME(s) \
 	((s).fore_color_name)
 #define SSET_FORE_COLOR_NAME(s,x) \
@@ -552,20 +546,36 @@
 	((s).icon_title_relief = (x))
 #define SGET_MIN_WINDOW_WIDTH(s) \
 	((s).min_window_width)
+#define SGET_MIN_WINDOW_WIDTH_IS_C(s) \
+	((s).min_window_width_is_c)
 #define SSET_MIN_WINDOW_WIDTH(s,x) \
 	((s).min_window_width = (x))
+#define SSET_MIN_WINDOW_WIDTH_IS_C(s,x) \
+	((s).min_window_width_is_c = (x))
 #define SGET_MAX_WINDOW_WIDTH(s) \
 	((s).max_window_width)
+#define SGET_MAX_WINDOW_WIDTH_IS_C(s) \
+	((s).max_window_width_is_c)
 #define SSET_MAX_WINDOW_WIDTH(s,x) \
 	((s).max_window_width = (x))
+#define SSET_MAX_WINDOW_WIDTH_IS_C(s,x) \
+	((s).max_window_width_is_c = (x))
 #define SGET_MIN_WINDOW_HEIGHT(s) \
 	((s).min_window_height)
+#define SGET_MIN_WINDOW_HEIGHT_IS_C(s) \
+	((s).min_window_height_is_c)
 #define SSET_MIN_WINDOW_HEIGHT(s,x) \
 	((s).min_window_height = (x))
+#define SSET_MIN_WINDOW_HEIGHT_IS_C(s,x) \
+	((s).min_window_height_is_c = (x))
 #define SGET_MAX_WINDOW_HEIGHT(s) \
 	((s).max_window_height)
+#define SGET_MAX_WINDOW_HEIGHT_IS_C(s) \
+	((s).max_window_height_is_c)
 #define SSET_MAX_WINDOW_HEIGHT(s,x) \
 	((s).max_window_height = (x))
+#define SSET_MAX_WINDOW_HEIGHT_IS_C(s,x) \
+	((s).max_window_height_is_c = (x))
 #define SGET_WINDOW_SHADE_STEPS(s) \
 	((s).shade_anim_steps)
 #define SSET_WINDOW_SHADE_STEPS(s,x) \
@@ -683,4 +693,4 @@ void free_icon_boxes(icon_boxes *ib);
 void style_destroy_style(style_id_t s_id);
 void print_styles(int verbose);
 
-#endif /* _STYLE_ */
+#endif /* FVWM_STYLE_H */
