@@ -10,8 +10,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 #include "config.h"
@@ -441,7 +440,7 @@ static int JmpArgs = 0;
 
 static Function *parse_function(char **line, char *pstop_char)
 {
-  Function *ftype = (Function *)safemalloc(sizeof(Function));
+  Function *ftype = fxmalloc(sizeof(Function));
   char *ptr, *name, *tok;
   int j, flag;
   FunctionType *builtin_functions_i;
@@ -656,7 +655,7 @@ Binding *ParseMouseEntry(char *tline)
 {
   char modifiers[20],*action,*token;
   Binding *new;
-  int button;
+  int button = -1;
   int n1=0,n2=0;
   int mods;
 
@@ -682,12 +681,11 @@ Binding *ParseMouseEntry(char *tline)
 
   modifiers_string_to_modmask(modifiers, &mods);
   if((mods & AnyModifier)&&(mods&(~AnyModifier))) {
-    ConsoleMessage("Binding specified AnyModifier and other modifers too. "
+    ConsoleMessage("Binding specified AnyModifier and other modifiers too. "
 		    "Excess modifiers will be ignored.");
   }
 
-  new = (Binding *)safemalloc(sizeof(Binding));
-  memset(new, 0, sizeof(Binding));
+  new = fxcalloc(1, sizeof(Binding));
   new->type = BIND_BUTTONPRESS;
   new->Button_Key = button;
   new->Modifier = mods;
@@ -737,7 +735,7 @@ static Binding *ParseKeyEntry(char *tline)
 
   modifiers_string_to_modmask(modifiers, &mods);
   if((mods & AnyModifier)&&(mods&(~AnyModifier))) {
-    ConsoleMessage("Binding specified AnyModifier and other modifers too. Excess modifiers will be ignored.");
+    ConsoleMessage("Binding specified AnyModifier and other modifiers too. Excess modifiers will be ignored.");
   }
 
   /*
@@ -753,7 +751,7 @@ static Binding *ParseKeyEntry(char *tline)
 
   XDisplayKeycodes(theDisplay, &min, &max);
   for (i=min; i<=max; i++) {
-    if (XKeycodeToKeysym(theDisplay, i, 0) == keysym) {
+    if (fvwm_KeycodeToKeysym(theDisplay, i, 0, 0) == keysym) {
       if (!func) {
 	func = parse_function_list(action);
 	if (!func) {
@@ -766,8 +764,7 @@ static Binding *ParseKeyEntry(char *tline)
 	actionstring = keystring = NULL;
       }
       temp = new;
-      new  = (Binding *)safemalloc(sizeof(Binding));
-      memset(new, 0, sizeof(Binding));
+      new  = fxcalloc(1, sizeof(Binding));
       new->type = BIND_KEYPRESS;
       new->Button_Key = i;
       new->key_name = keystring;
@@ -797,8 +794,7 @@ static Binding *ParseSimpleEntry(char *tline)
 		return NULL;
 	}
 
-	new = (Binding *)safemalloc(sizeof(Binding));
-	memset(new, 0, sizeof(Binding));
+	new = fxcalloc(1, sizeof(Binding));
 	new->type = BIND_KEYPRESS;
 	new->key_name = "select";
 	new->Action = stripcpy(tline);
@@ -845,13 +841,6 @@ static int GetConfigLineWrapper(int *fd, char **tline)
 		{
 			LoadColorset(&(*tline)[8]);
 		}
-		else if (strncasecmp(
-			*tline, XINERAMA_CONFIG_STRING,
-			sizeof(XINERAMA_CONFIG_STRING) - 1) == 0)
-		{
-			FScreenConfigureModule(
-			(*tline) + sizeof(XINERAMA_CONFIG_STRING) - 1);
-		}
 		else if (strncasecmp(*tline, "IgnoreModifiers", 15) == 0)
 		{
 			sscanf((*tline) + 16, "%d", &mods_unused);
@@ -1144,6 +1133,86 @@ static void handle_button_config(int manager, int context, char *option)
 		copy_string(&globals.managers[id].backColorName[context], p));
 }
 
+static void handle_resolution_config(int man, char *line)
+{
+	char *token = NULL, *nline;
+	int n, type;
+	Resolution r;
+
+	/* Reset to initial state of no filters */
+	r.type = SHOW_ALL;
+	r.invert = false;
+	r.desk_n = -1;
+	r.pagex_n = -1;
+	r.pagey_n = -1;
+
+	/* Apply filters */
+	while ((token = PeekToken(line, &line)) != NULL)
+	{
+		/* Since multiple filters can be sent, ensure
+		 * only the last one takes precedence.
+		 */
+		if (!strcasecmp(token, "invert")) {
+			r.invert = true;
+			continue;
+		} else if (!strcasecmp(token, "desk")) {
+			type = SHOW_DESK;
+			r.type = (r.type | SHOW_DESK) & ~(NO_SHOW_DESK);
+		} else if (!strcasecmp(token, "page")) {
+			type = SHOW_PAGE;
+			r.type = (r.type | SHOW_PAGE) & ~(NO_SHOW_PAGE);
+		} else if (!strcasecmp(token, "screen")) {
+			type = SHOW_SCREEN;
+			r.type = (r.type | SHOW_SCREEN) & ~(NO_SHOW_SCREEN);
+		} else if (!strcasecmp(token, "!desk")) {
+			type = SHOW_DESK;
+			r.type = (r.type | NO_SHOW_DESK) & ~(SHOW_DESK);
+		} else if (!strcasecmp(token, "!page")) {
+			type = SHOW_PAGE;
+			r.type = (r.type | NO_SHOW_PAGE) & ~(SHOW_PAGE);
+		} else if (!strcasecmp(token, "!screen")) {
+			type = SHOW_SCREEN;
+			r.type = (r.type | NO_SHOW_SCREEN) & ~(SHOW_SCREEN);
+		} else {
+			continue;
+		}
+		token = PeekToken(line, &nline);
+
+		switch (type)
+		{
+		case SHOW_DESK:
+			if (token && extract_int(token, &n) && n >= 0) {
+				r.desk_n = n;
+				line = nline;
+			}
+			break;
+		case SHOW_PAGE:
+			if (token && extract_int(token, &n) && n >= 0) {
+				r.pagex_n = n;
+				line = nline;
+			}
+			token = PeekToken(line, &nline);
+			if (token && extract_int(token, &n) && n >= 0) {
+				r.pagey_n = n;
+				line = nline;
+			}
+			break;
+		case SHOW_SCREEN:
+			if (token == NULL) {
+				SET_MANAGER(man, scr, NULL);
+				break;
+			}
+			struct monitor *mon = monitor_resolve_name(token);
+			SET_MANAGER(man, scr, (char *)mon->si->name);
+			line = nline;
+			break;
+		default:
+			break;
+		}
+	}
+	SET_MANAGER(man, res, r);
+}
+
 static void add_weighted_sort(WinManager *man, WeightedSort *weighted_sort)
 {
 	WeightedSort *p;
@@ -1151,9 +1220,9 @@ static void add_weighted_sort(WinManager *man, WeightedSort *weighted_sort)
 	if (man->weighted_sorts_len == man->weighted_sorts_size)
 	{
 		man->weighted_sorts_size += 16;
-		man->weighted_sorts = (WeightedSort *)saferealloc(
+		man->weighted_sorts = fxrealloc(
 			(char *)man->weighted_sorts,
-			man->weighted_sorts_size * sizeof(WeightedSort));
+			man->weighted_sorts_size, sizeof(WeightedSort));
 	}
 	p = &man->weighted_sorts[man->weighted_sorts_len];
 	p->resname = NULL;
@@ -1186,7 +1255,6 @@ void read_in_resources(void)
   int i, n, manager;
   char *option1;
   Binding *binding = NULL;
-  Resolution r;
   Reverse rv;
 
   InitGetConfigLine(fvwm_fd, Module);
@@ -1416,7 +1484,7 @@ void read_in_resources(void)
 	  continue;
 	}
 	do {
-	  ConsoleDebug(CONFIG, "dont show: %s\n", token);
+	  ConsoleDebug(CONFIG, "don't show: %s\n", token);
 	  if (manager == -1) {
 	    int i;
 	    for (i = 0; i < globals.num_managers; i++)
@@ -1608,7 +1676,7 @@ void read_in_resources(void)
 	DoGetNextToken(p, &token, NULL, ",", NULL);
 	if (!token)
 	  {
-	    token = (char *)safemalloc(1);
+	    token = fxmalloc(1);
 	    *token = 0;
 	  }
 
@@ -1632,7 +1700,7 @@ void read_in_resources(void)
 	DoGetNextToken(p, &token, NULL, ",", NULL);
 	if (!token)
 	  {
-	    token = (char *)safemalloc(1);
+	    token = fxmalloc(1);
 	    *token = 0;
 	  }
 
@@ -1651,33 +1719,8 @@ void read_in_resources(void)
 		     copy_string(&globals.managers[id].geometry_str, p));
       }
       else if (!strcasecmp(option1, "resolution")) {
-	p = read_next_cmd(READ_ARG);
-	if (!p) {
-	  ConsoleMessage("Bad line: %s\n", current_line);
-	  continue;
-	}
-	ConsoleDebug(CONFIG, "resolution: %s\n", p);
-	if (!strcasecmp(p, "global"))
-	  r = SHOW_GLOBAL;
-	else if (!strcasecmp(p, "desk"))
-	  r = SHOW_DESKTOP;
-	else if (!strcasecmp(p, "page"))
-	  r = SHOW_PAGE;
-	else if (!strcasecmp(p, "screen"))
-	  r = SHOW_SCREEN;
-	else if (!strcasecmp(p, "!desk"))
-	  r = NO_SHOW_DESKTOP;
-	else if (!strcasecmp(p, "!page"))
-	  r = NO_SHOW_PAGE;
-	else if (!strcasecmp(p, "!screen"))
-	  r = NO_SHOW_SCREEN;
-	else {
-	  ConsoleMessage("Bad line: %s\n", current_line);
-	  ConsoleMessage("What kind of resolution is this?\n");
-	  continue;
-	}
-
-	SET_MANAGER(manager, res, r);
+	p = read_next_cmd(READ_REST_OF_LINE);
+	handle_resolution_config(manager, p);
       }
       else if (!strcasecmp(option1, "reverse")) {
 	p = read_next_cmd(READ_ARG);
@@ -1859,7 +1902,7 @@ void read_in_resources(void)
 	DoGetNextToken(p, &token, NULL, ",", NULL);
 	if (!token)
 	  {
-	    token = (char *)safemalloc(1);
+	    token = fxmalloc(1);
 	    *token = 0;
 	  }
 
@@ -1877,7 +1920,7 @@ void read_in_resources(void)
 	DoGetNextToken(p, &token, NULL, ",", NULL);
 	if (!token)
 	  {
-	    token = (char *)safemalloc(1);
+	    token = fxmalloc(1);
 	    *token = 0;
 	  }
 
@@ -1888,6 +1931,9 @@ void read_in_resources(void)
       else if (!strcasecmp(option1, "iconButton")) {
 	handle_button_config(manager, ICON_CONTEXT, option1);
       }
+      else if (!strcasecmp(option1, "iconandselectButton")) {
+	handle_button_config(manager, ICON_SELECT_CONTEXT, option1);
+      }
       else if (!strcasecmp(option1, "plainButton")) {
 	handle_button_config(manager, PLAIN_CONTEXT, option1);
       }
@@ -1987,6 +2033,20 @@ void read_in_resources(void)
 	SET_MANAGER(manager, colorsets[ICON_CONTEXT], n);
 	AllocColorset(n);
       }
+      else if (!strcasecmp(option1, "iconandselectcolorset")) {
+	p = read_next_cmd(READ_ARG);
+	if (!p) {
+	  ConsoleMessage("Bad line: %s\n", current_line);
+	  continue;
+	}
+	if (extract_int(p, &n) == 0) {
+	  ConsoleMessage("This is not a number: %s\n", p);
+	  ConsoleMessage("Bad line: %s\n", current_line);
+	  continue;
+	}
+	SET_MANAGER(manager, colorsets[ICON_SELECT_CONTEXT], n);
+	AllocColorset(n);
+      }
       else if (!strcasecmp(option1, "usewinlist")) {
 	p = read_next_cmd(READ_ARG);
 	if (!p) {
@@ -2123,7 +2183,7 @@ void read_in_resources(void)
 	DoGetNextToken(p, &token, NULL, ",", NULL);
 	if (!token)
 	  {
-	    token = (char *)safemalloc(1);
+	    token = fxmalloc(1);
 	    *token = 0;
 	  }
 
@@ -2289,38 +2349,7 @@ void process_dynamic_config_line(char *line)
 	/* currently supports only "resolution" and "tips" */
 	if (strcasecmp(token, "resolution") == 0)
 	{
-		int value;
-
-		free(token);
-		line = GetNextToken(line, &token);
-		if (!token)
-		{
-			return;
-		}
-		if (!strcasecmp(token, "global"))
-			value = SHOW_GLOBAL;
-		else if (!strcasecmp(token, "desk"))
-			value = SHOW_DESKTOP;
-		else if (!strcasecmp(token, "page"))
-			value = SHOW_PAGE;
-		else if (!strcasecmp(token, "screen"))
-			value = SHOW_SCREEN;
-		else if (!strcasecmp(token, "!desk"))
-			value = NO_SHOW_DESKTOP;
-		else if (!strcasecmp(token, "!page"))
-			value = NO_SHOW_PAGE;
-		else if (!strcasecmp(token, "!screen"))
-			value = NO_SHOW_SCREEN;
-		else
-		{
-			fprintf(
-				stderr, "%s: unknown resolution %s.\n",
-				MyName, token);
-			free(token);
-			return;
-		}
-
-		SET_MANAGER(manager, res, value);
+		handle_resolution_config(manager, line);
 		remanage_winlist();
 	}
 	else if (strcasecmp(token, "tips") == 0)
